"""Base functionality for fancy validation."""
from operator import attrgetter

from flatland.schema.util import find_i18n_function


N_ = lambda translatable: translatable
P_ = lambda *translatable: translatable
_ugettext_finder = attrgetter('ugettext')
_ungettext_finder = attrgetter('ungettext')


class Validator(object):
    """Base class for fancy validators."""

    def __init__(self, **kw):
        """Construct a validator.

        :param \*\*kw: override any extant class attribute on this instance.

        """
        cls = type(self)
        for attr, value in kw.iteritems():
            if hasattr(cls, attr):
                setattr(self, attr, value)
            else:
                raise TypeError("%s has no attribute %r, can not override." % (
                    cls.__name__, attr))

    def __call__(self, element, state):
        """Adapts Validator to the Element.validate callable interface."""
        return self.validate(element, state)

    def validate(self, element, state):
        """Validate an element returning True if valid.

        Abstract.

        :param element:
          an :class:`~flatland.schema.base.Element` instance.

        :param state:
          an arbitrary object.  Supplied by
          :meth:`Element.validate <flatland.schema.base.Element.validate>`.

        :returns: True if valid

        """
        return False

    def note_error(self, element, state, key=None, message=None, **info):
        """Record a validation error message on an element.

        :param element:
          An :class:`~flatland.schema.base.Element` instance.

        :param state:
          an arbitrary object.  Supplied by :meth:`Element.validate
          <flatland.schema.base.Element.validate>`.

        :param key: semi-optional, default None.
          The name of a message-holding attribute on this instance.  Will be
          used to ``message = getattr(self, key)``.

        :param message: semi-optional, default None.  A validation
          message.  Use to provide a specific message rather than look
          one up by *key*.

        :param \*\*info: optional.
          Additional data to make available to validation message
          string formatting.

        :returns: False

        Either *key* or *message* is required.  The message will have
        formatting expanded by :meth:`expand_message` and be appended to
        :attr:`element.errors <flatland.schema.base.Element.errors>`.

        Always returns False.  This enables a convenient shorthand when
        writing validators:

        .. testcode::

          from flatland.validation import Validator

          class MyValidator(Validator):
              my_message = 'Oh noes!'

              def validate(self, element, state):
                  if not element.value:
                      return self.note_error(element, state, 'my_message')
                  else:
                      return True

        .. testcode:: :hide:

          from flatland import String
          el = String()
          v = MyValidator()
          assert not v.validate(el, None)
          assert el.errors == ['Oh noes!']
          el.set('foo')
          assert v.validate(el, None)
          assert el.errors == ['Oh noes!']

        """
        message = message or getattr(self, key)
        if message:
            element.add_error(
                self.expand_message(element, state, message, **info))
        return False

    def note_warning(self, element, state, key=None, message=None, **info):
        """Record a validation warning message on an element.

        :param element:
          An :class:`~flatland.schema.base.Element` instance.

        :param state:
          an arbitrary object.  Supplied by :meth:`Element.validate
          <flatland.schema.base.Element.validate>`.

        :param key: semi-optional, default None.
          The name of a message-holding attribute on this instance.  Will be
          used to ``message = getattr(self, key)``.

        :param message: semi-optional, default None.  A validation
          message.  Use to provide a specific message rather than look
          one up by *key*.

        :param \*\*info: optional.
          Additional data to make available to validation message
          string formatting.

        :returns: False

        Either *key* or *message* is required.  The message will have
        formatting expanded by :meth:`expand_message` and be appended to
        :attr:`element.warnings <flatland.schema.base.Element.warnings>`.

        Always returns False.
        """
        message = message or getattr(self, key)
        if message:
            element.add_warning(
                self.expand_message(element, state, message, **info))
        return False

    def find_transformer(self, type, element, state, message):
        """Locate a message-transforming function, such as ugettext.

        Returns None or a callable.  The callable must return a
        message.  The call signature of the callable is expected to
        match ``ugettext`` or ``ungettext``:

        - If *type* is 'ugettext', the callable should take a message
          as a positional argument.

        - If *type* is 'ungettext', the callable should take three
          positional arguments: a message for the singular form, a
          message for the plural form, and an integer.

        Subclasses may override this method to provide advanced
        message transformation and translation functionality, on a
        per-element or per-message granularity if desired.

        The default implementation uses the following logic to locate
        a transformer:

        1.  If *state* has an attribute or item named *type*, return that.

        2.  If the *element* or any of its parents have an attribute
            named *type*, return that.

        3.  If the schema of *element* or the schema of any of its
            parents have an attribute named *type*, return that.

        4.  If *type* is in ``__builtin__``, return that.

        5.  Otherwise return ``None``.

        """
        if hasattr(state, type):
            return getattr(state, type)
        if hasattr(state, '__getitem__'):
            try:
                return state[type]
            except KeyError:
                pass

        if type == 'ugettext':
            finder = _ugettext_finder
        elif type == 'ungettext':
            finder = _ungettext_finder
        else:
            raise RuntimeError("Unknown transformation %r" % type)
        return find_i18n_function(element, finder)

    def expand_message(self, element, state, message, **extra_format_args):
        """Apply formatting to a validation message.

        :param element:
          an :class:`~flatland.schema.base.Element` instance.

        :param state:
          an arbitrary object.  Supplied by
          :meth:`Element.validate <flatland.schema.base.Element.validate>`.

        :param message: a string, 3-tuple or callable.
          If a 3-tuple, must be of the form ('single form', 'plural form',
          n_key).

          If callable, will be called with 2 positional arguments (*element*,
          *state*) and must return a string or 3-tuple.

        :param \*\*extra_format_args: optional.
          Additional data to make available to validation message
          string formatting.

        :returns: the formatted string

        See :ref:`Message Templating`, :ref:`Message Pluralization` and
        :ref:`Message Internationalization` for full information on how
        messages are expanded.

        """
        if callable(message):
            message = message(element, state)

        ugettext = self.find_transformer('ugettext', element, state, message)

        format_map = as_format_mapping(
            extra_format_args, state, self, element,
            transform=ugettext)

        if isinstance(message, tuple):
            ungettext = self.find_transformer(
                'ungettext', element, state, message)

            single, plural, n_key = message
            try:
                n = format_map[n_key]
                try:
                    n = int(n)
                except TypeError:
                    pass
            except KeyError:
                n = n_key

            if ungettext:
                message = ungettext(single, plural, n)
            else:
                if ugettext:
                    single = ugettext(single)
                    plural = ugettext(plural)
                message = single if n == 1 else plural
        elif ugettext:
            message = ugettext(message)

        return message % format_map


class as_format_mapping(object):
    """A unified, optionally transformed, mapping view over multiple instances.

    Allows regular instance attributes to be accessed by "%(attrname)s" in
    string formats.  Dictionaries may be included as well.  Optionally
    passes values through a ``transform`` (such ``gettext``) before
    returning.

    """

    __slots__ = 'targets', 'transform'

    def __init__(self, *targets, **kw):
        self.targets = [t for t in targets if t is not None]
        self.transform = kw.pop('transform', None)
        if kw:
            raise TypeError('unexpected keyword argument')

    def __getitem__(self, item):
        for target in self.targets:
            # try target[item] first
            if hasattr(target, '__getitem__'):
                try:
                    value = target[item]
                    break
                except (LookupError, TypeError):
                    pass
            # then target.item
            try:
                value = getattr(target, item)
                break
            except AttributeError:
                pass
        else:
            # not found on any target
            raise KeyError(item)

        if self.transform:
            return self.transform(value)
        else:
            return value

    def __contains__(self, item):
        try:
            self[item]
            return True
        except KeyError:
            return False

    def __iter__(self):
        keys = set()
        for target in self.targets:
            keys |= set(dir(target))
        return iter(keys)



import re
from flatland.util import re_ucompile
from flatland.validation.base import N_, Validator


class NANPphone(Validator):
    invalid = N_(u'The %(label)s is not valid.')

    noext = N_(u'%(label)s must be a ten-digit phone number '
               u'without extension.')

    re_cleaner = re_ucompile(ur'\D')
    re_lowbit = re.compile('^[0-9]+$')
    fmt_line = u'(%03i) %03i-%04i'
    fmt_ext = fmt_line + ' x%i'

    def __init__(self, extensions=False, **kw):
        Validator.__init__(self, **kw)
        self.extensions = extensions

    def validate(self, element, state):
        value = element.u if element.value is None else element.value
        value = self.re_cleaner.sub('', value)

        if value == u'':
            return self.note_error(element, state, 'invalid')

        if not self.re_lowbit.match(value):
            # Translate to single-byte numbers.
            value = unicode(str(long(value)))

        if len(value) < 10:
            return self.note_error(element, state, 'invalid')
        elif len(value) == 10 or self.extensions == 'strip':
            value = self.fmt_line % (int(value[0:3]),
                                     int(value[3:6]),
                                     int(value[6:10]))
        else:
            if not self.extensions:
                return self.note_error(element, state, 'noext')
            value = self.fmt_ext % (value[0:3], value[3:6], value[6:10],
                                    value[10:])

        element.set(value)
        return True



"""Schemas for structured data."""
from flatland.exc import AdaptationError
from flatland.schema import Array, Boolean, Compound, Constrained, Container,\
        Date, DateTime, DateYYYYMMDD, Decimal, Dict, Element, Enum, Float, Form,\
        Integer, JoinedString, List, Long, Mapping, MultiValue, Number,\
        Properties, Ref, Scalar, Sequence, Skip, SkipAll, SkipAllFalse,\
        SparseDict, String, Time, Unevaluated, Unset

'''
from flatland.util.deferred import deferred_module


deferred_module.shadow(
    'flatland',
    {'exc': ('AdaptationError',),
     'schema': ('Array',
                'Boolean',
                'Compound',
                'Constrained',
                'Container',
                'Date',
                'DateTime',
                'DateYYYYMMDD',
                'Decimal',
                'Dict',
                'Element',
                'Enum',
                'Float',
                'Form',
                'Integer',
                'JoinedString',
                'List',
                'Long',
                'Mapping',
                'MultiValue',
                'Number',
                'Properties',
                'Ref',
                'Scalar',
                'Sequence',
                'Skip',
                'SkipAll',
                'SkipAllFalse',
                'SparseDict',
                'String',
                'Time',
                'Unevaluated',
                'Unset',
                ),
     'signals': (),
     'util': ('Unspecified', 'class_cloner',),
     'validation': (),
     },
    __version__='dev')
'''
__version__='fix-imports'



from flatland import (
    Integer,
    List,
    String,
    Unset,
)
from flatland.schema.base import Unspecified

from tests._util import eq_


def test_set_flat_linear():
    pairs = [(u'l_0_i', 1), (u'l_1_i', 2), (u'l_2_i', 3)]

    schema = List.named(u'l').of(Integer.named(u'i'))
    el = schema.from_flat(pairs)

    eq_(len(el), len(pairs))
    eq_(el.value, list(pair[1] for pair in pairs))


def test_set_flat_miss():
    pairs = [(u'l_galump', u'3'), (u'l_snorgle', u'4')]

    schema = List.named(u'l').of(Integer.named(u'i'))
    el = schema.from_flat(pairs)

    eq_(len(el), 0)
    eq_(el.value, [])


def test_set_flat_scalar():
    pairs = [(u'l', u'1')]

    schema = List.named(u'l').of(Integer.named(u'i'))
    el = schema()
    canary = []
    def setter(self, value):
        canary.append(value)
        return type(el).set(self, value)
    el.set = setter.__get__(el, type(el))

    el.set_flat(pairs)
    eq_(len(el), 0)
    eq_(el.value, [])
    eq_(canary, [])


def test_set_flat_pruned():
    # pruned won't insert empty elements for a skipped index or empty rhs
    pairs = [(u'l_0_i', u'0'), (u'l_2_i', u''), (u'l_3_i', u'3')]

    schema = List.named(u'l').of(Integer.named(u'i'))
    el = schema.from_flat(pairs)

    eq_(len(el), 2)
    eq_(el.value, [0, 3])

    schema2 = schema.using(maximum_set_flat_members=1)
    el = schema2.from_flat(pairs)

    eq_(len(el), 1)
    eq_(el.value, [0])


def test_set_flat_unpruned():
    pairs = [(u'l_0_i', u'0'), (u'l_2_i', u''), (u'l_3_i', u'3')]

    schema = List.named(u'l').of(Integer.named(u'i')).using(prune_empty=False)
    el = schema.from_flat(pairs)

    eq_(len(el), 4)
    eq_(el.value, [0, None, None, 3])

    schema2 = schema.using(maximum_set_flat_members=2)
    el = schema2.from_flat(pairs)

    eq_(len(el), 2)
    eq_(el.value, [0, None])


def _assert_set_flat(schema, pairs, bogus=[]):
    el = schema.from_flat(pairs + bogus)
    eq_(len(el), len(pairs))
    eq_(el.value, list(pair[1] for pair in pairs))
    eq_(el.flatten(), pairs)


def test_set_flat_unnamed_child():
    schema = List.named(u's').of(String)
    pairs = [(u's_0', u'abc'), (u's_1', u'def')]
    bogus = [(u's', u'xxx')]

    _assert_set_flat(schema, pairs, bogus)


def test_set_flat_anon_list_named_child():
    schema = List.of(String.named(u's'))
    pairs = [(u'0_s', u'abc'), (u'1_s', u'def')]
    bogus = [(u's', u'xxx'), (u'0', u'yyy')]

    _assert_set_flat(schema, pairs, bogus)


def test_set_flat_fully_anonymous():
    schema = List.of(String)
    pairs = [(u'0', u'abc'), (u'1', u'def')]
    bogus = [(u'x', u'xxx')]

    _assert_set_flat(schema, pairs, bogus)


def test_set_flat_anonymous_dict():
    pairs = ((u'l_0_x', u'x0'), (u'l_0_y', u'y0'),
             (u'l_1_x', u'x1'), (u'l_1_y', u'y1'),
             (u'l_2_x', u'x2'), )

    schema = List.named(u'l').of(String.named(u'x'), String.named(u'y'))
    el = schema.from_flat(pairs)

    eq_(len(el), 3)
    eq_(el[0].value, dict((k[-1], v) for k, v in pairs[0:2]))
    eq_(el[1].value, dict((k[-1], v) for k, v in pairs[2:4]))
    eq_(el[2].value, {u'x': u'x2', u'y': None})


def test_set_flat_doubly_anonymous_dict():
    pairs = ((u'0_x', u'x0'), (u'0_y', u'y0'),
             (u'1_x', u'x1'), (u'1_y', u'y1'),
             (u'2_x', u'x2'), )

    schema = List.of(String.named(u'x'), String.named(u'y'))
    el = schema.from_flat(pairs)

    eq_(len(el), 3)
    eq_(el[0].value, dict((k[-1], v) for k, v in pairs[0:2]))
    eq_(el[1].value, dict((k[-1], v) for k, v in pairs[2:4]))
    eq_(el[2].value, {u'x': u'x2', u'y': None})


def test_set_default_int():

    def factory(count, **kw):
        return List.named(u'l').using(default=count, **kw).of(
            String.named(u's').using(default=u'val'))

    schema = factory(3)
    el = schema()
    eq_(len(el), 0)
    eq_(el.value, [])

    el = schema()
    el.set_default()
    eq_(len(el), 3)
    eq_(el.value, [u'val'] * 3)

    el.append(None)
    eq_(len(el), 4)
    eq_(el[-1].value, None)
    el[-1].set_default()
    eq_(el[-1].value, u'val')

    el = schema([u'a', u'b'])
    eq_(len(el), 2)
    eq_(el.value, [u'a', u'b'])
    el.set_default()
    eq_(len(el), 3)
    eq_(el.value, [u'val'] * 3)

    schema0 = factory(0)

    el = schema0()
    el.set_default()
    eq_(len(el), 0)
    eq_(el.value, [])

    def calculated_default(element):
        assert isinstance(element, List)
        return 2

    schemaf = List.named(u'l').using(default_factory=calculated_default).of(
        String.named(u's').using(default=u'val'))

    el = schemaf()
    el.set_default()
    eq_(len(el), 2)
    eq_(el.value, [u'val'] * 2)


def test_set_default_value():

    def factory(default, **kw):
        return List.using(default=default, **kw).of(
            String.using(default=u'val'))

    schema = factory([u'a', u'b'])
    el = schema()
    eq_(len(el), 0)
    eq_(el.value, [])

    el = schema()
    el.set_default()
    eq_(len(el), 2)
    eq_(el.value, [u'a', u'b'])

    el = schema([u'c', u'd'])
    eq_(el.value, [u'c', u'd'])
    el.set_default()
    eq_(el.value, [u'a', u'b'])

    schema0 = factory([])
    el = schema0()
    el.set_default()
    eq_(len(el), 0)
    eq_(el.value, [])


def test_set_default_none():

    def factory(default, **kw):
        return List.using(default=default, **kw).of(
            String.using(default=u'val'))

    for default in None, Unspecified:
        schema = factory(default)
        el = schema()
        eq_(el.value, [])

        el = schema()
        el.set_default()
        eq_(el.value, [])

    # TODO: exercising this here (set_default of None doesn't reset
    # the value), but unsure if this is the correct behavior
    schema = factory(None)
    el = schema([u'a', u'b'])
    eq_(el.value, [u'a', u'b'])
    el.set_default()
    eq_(el.value, [u'a', u'b'])


def test_set():
    schema = List.of(Integer)

    el = schema()
    assert list(el) == []

    el = schema()
    assert not el.set(1)
    assert el.value == []

    el = schema()
    assert not el.set(None)
    assert el.value == []

    el = schema()
    assert el.set(range(3))
    assert el.value == [0, 1, 2]

    el = schema()
    assert el.set(xrange(3))
    assert el.value == [0, 1, 2]

    el = schema([0, 1, 2])
    assert el.value == [0, 1, 2]

    el = schema()
    el.extend([1, 2, 3])
    assert el.value == [1, 2, 3]
    el.set([4, 5, 6])
    assert el.value == [4, 5, 6]
    assert el.set([])
    assert el.value == []


def test_raw():
    schema = List.of(Integer)
    el = schema()
    assert el.raw is Unset

    el = schema(u'foo')
    assert el.raw == u'foo'

    el = schema([1, 2, 3])
    assert el.raw == [1, 2, 3]

    el = schema((1, 2, 3))
    assert el.raw == (1, 2, 3)

    el = schema({u'x': u'bar'})
    assert el.raw == {u'x': u'bar'}


def test_access():
    pairs = ((u'l_0_i', u'10'), (u'l_1_i', u'11'), (u'l_2_i', u'12'),)

    schema = List.named(u'l').of(Integer.named(u'i'))
    el = schema.from_flat(pairs)

    elements = list(Integer.named(u'i')(val)
                    for val in (u'10', u'11', u'12'))

    assert len(el) == 3
    assert el[0] == elements[0]
    assert el[1] == elements[1]
    assert el[2] == elements[2]

    assert el[0].value == 10

    assert el[:0] == elements[:0]
    assert el[:1] == elements[:1]
    assert el[0:5] == elements[0:5]
    assert el[-2:-1] == elements[-2:-1]

    assert el[0] in el
    assert elements[0] in el
    assert u'10' in el
    assert 10 in el

    assert el.count(elements[0]) == 1
    assert el.count(u'10') == 1
    assert el.count(10) == 1

    assert el.index(elements[0]) == 0
    assert el.index(u'10') == 0
    assert el.index(10) == 0


def test_mutation():
    schema = List.named(u'l').of(Integer.named(u'i'))
    el = schema()

    new_element = Integer.named(u'i')

    def order_ok():
        slot_names = list(_.name for _ in el._slots)
        for idx, name in enumerate(slot_names):
            assert name == str(idx).decode('ascii')

    assert not el
    order_ok()

    # FIXME:? seems to want parsable data, not elements
    el.append(new_element(u'0'))
    assert el.value == [0]
    order_ok()

    el.append(u'123')
    assert el.value == [0, 123]
    order_ok()

    el.extend([u'4', u'5'])
    assert el.value == [0, 123, 4, 5]
    order_ok()

    el[0] = u'3'
    assert el.value == [3, 123, 4, 5]
    order_ok()

    el.insert(0, u'2')
    assert el.value == [2, 3, 123, 4, 5]
    order_ok()

    v = el.pop()
    assert v.value == 5
    assert not v.parent
    order_ok()

    v = el.pop(0)
    assert v.value == 2
    assert el.value == [3, 123, 4]
    order_ok()

    el.remove(u'3')
    assert el.value == [123, 4]
    order_ok()

    del el[:]
    assert el.value == []
    order_ok()


def test_mutate_slices():
    schema = List.named(u'l').of(Integer.named(u'i'))
    el = schema()

    canary = []

    el.extend([u'3', u'4'])
    canary.extend([3, 4])

    el[0:1] = [u'1', u'2', u'3']
    canary[0:1] = [1, 2, 3]
    eq_(el.value, [1, 2, 3, 4])
    eq_(canary, [1, 2, 3, 4])

    del el[2:]
    del canary[2:]
    assert el.value == [1, 2]
    assert canary == [1, 2]


def test_reverse():
    schema = List.named(u'l').of(Integer.named(u'i'))
    el = schema([2, 1])
    assert el.flatten() == [(u'l_0_i', u'2'), (u'l_1_i', u'1')]

    el.reverse()
    assert el.value == [1, 2]
    assert el.flatten() == [(u'l_0_i', u'1'), (u'l_1_i', u'2')]


def test_sort():
    schema = List.named(u'l').of(Integer.named(u'i'))
    el = schema([2, 1])

    el.sort(key=lambda el: el.value)
    assert el.value == [1, 2]
    assert el.flatten() == [(u'l_0_i', u'1'), (u'l_1_i', u'2')]

    el.sort(key=lambda el: el.value, reverse=True)
    assert el.value == [2, 1]
    assert el.flatten() == [(u'l_0_i', u'2'), (u'l_1_i', u'1')]


def test_slots():
    schema = List.named(u'l').of(Integer.named(u'i'))
    el = schema([1, 2])

    assert len(list(el._slots)) == 2
    for slot in el._slots:
        # don't really care what it says, just no crashy.
        assert repr(slot)

    assert [slot.value for slot in el._slots] == [1, 2]


def test_u():
    schema = List.of(String)
    el = schema([u'x', u'x'])
    eq_(el.u, u"[u'x', u'x']")


def test_value():
    schema = List.of(String)
    el = schema([u'x', u'x'])
    eq_(el.value, [u'x', u'x'])



# -*- coding: utf-8; fill-column: 78 -*-
"""Network address and URL validation."""
import re
import urlparse

from base import N_, Validator


class IsEmail(Validator):
    """Validates email addresses.

    The default behavior takes a very permissive stance on allowed characters
    in the **local-part** and a relatively strict stance on the **domain**.
    Given **local-part@domain**:

    - **local-part** must be present and contain at least one non-whitespace
      character.  Any character is permitted, including international
      characters.

    - **domain** must be preset, less than 253 characters and each
      dot-separated component must be 63 characters or less.  **domain** may
      contain non-ASCII international characters, and will be converted to IDN
      representation before length assertions are applied.  No top level
      domain validations are applied.

    **Attributes**

    .. attribute:: nonlocal

      Default ``True``.  When true, require at minimum two domain name
      components and reject local email addresses such as
      ``postmaster@localhost`` or ``user@workstation``.

    .. attribute:: local_part_pattern

      No default.  If present, a compiled regular expression that will be
      matched to the **local-part**.  Override this to implement more
      stringent checking such as RFC-compliant addresses.

    .. attribute:: domain_pattern

      Defaults to a basic domain-validating regular expression with no notion
      of valid top level domains.  Override this to require certain TLDs (or
      alternately and more simply, add another validator to your chain that
      checks the endings of the string against your list of TLDs.)

      The default pattern rejects the valid but obscure quoted IP-address form
      (``[1.2.3.4]``).

    **Messages**

    .. attribute:: invalid

      Emitted if the email address is not valid.

    """

    invalid = N_(u'%(label)s is not a valid email address.')

    nonlocal = True

    local_part_pattern = None

    domain_pattern = re.compile(r'^(?:[a-z0-9\-]+\.)*[a-z0-9\-]+$',
                                re.IGNORECASE)

    def validate(self, element, state):
        addr = element.u
        if addr.count(u'@') != 1:
            return self.note_error(element, state, 'invalid')

        local_part, domain = addr.split(u'@')
        if not local_part or local_part.isspace():
            return self.note_error(element, state, 'invalid')

        # optional local part validation
        if (self.local_part_pattern and
            not self.local_part_pattern.match(local_part)):
            return self.note_error(element, state, 'invalid')

        try:
            # convert domain to ascii
            domain = domain.encode('idna')
        except UnicodeError:
            return self.note_error(element, state, 'invalid')

        if len(domain) > 253:
            return self.note_error(element, state, 'invalid')

        if not self.domain_pattern.match(domain):
            return self.note_error(element, state, 'invalid')

        labels = domain.split('.')
        if len(labels) == 1 and self.nonlocal:
            return self.note_error(element, state, 'invalid')

        if not all(len(label) < 64 for label in labels):
            return self.note_error(element, state, 'invalid')

        return True


# ordered generic URL part names according to urlparse
_url_parts = ['scheme', 'netloc', 'path', 'params', 'query', 'fragment']

class URLValidator(Validator):
    """A general URL validator.

    Validates that a URL is well-formed and may optionally restrict
    the set of valid schemes and other URL components.

    **Attributes**

    .. attribute:: allowed_schemes

      Restrict URLs to just this sequence of named schemes, or allow
      all schemes with ('*',).  Defaults to all schemes.  Example::

        allowed_schemes = ('http', 'https', 'ssh')

    .. attribute:: allowed_parts

      A sequence of 0 or more part names in :mod:`urlparse`'s
      vocabulary::

        'scheme', 'netloc', 'path', 'params', 'query', 'fragment'

      Defaults to all parts allowed.

    .. attribute:: urlparse

      By default the :mod:`urlparse` module, but may be replaced by
      any object that implements :func:`urlparse.urlparse` and
      :func:`urlparse.urlunparse`.

    **Messages**

    .. attribute:: bad_format

      Emitted for an unparseable URL.

    .. attribute:: blocked_scheme

      Emitted if the URL ``scheme:`` is not present in
      :attr:`allowed_schemes`.

    .. attribute:: blocked_part

      Emitted if the URL has a component not present in
      :attr:`allowed_parts`.

    """

    bad_format = N_("%(label)s is not a valid URL.")
    blocked_scheme = N_("%(label)s is not a valid URL.")
    blocked_part = N_("%(label)s is not a valid URL.")

    allowed_schemes = ('*',)
    allowed_parts = set(_url_parts)
    urlparse = urlparse

    def validate(self, element, state):
        if element.value is None:
            return self.note_error(element, state, 'bad_format')

        try:
            url = self.urlparse.urlparse(element.value.strip())
        except Exception:
            return self.note_error(element, state, 'bad_format')

        scheme_name = url.scheme
        if scheme_name == u'':
            return self.note_error(element, state, 'blocked_scheme')
        elif self.allowed_schemes != ('*',):
            if scheme_name not in self.allowed_schemes:
                return self.note_error(element, state, 'blocked_scheme')

        for part in _url_parts:
            if (part not in self.allowed_parts and
                getattr(url, part) != ''):
                return self.note_error(element, state, 'blocked_part')
        return True


class HTTPURLValidator(Validator):
    """Validates ``http`` and ``https`` URLs.

    Validates that an ``http``-like URL is well-formed and may
    optionally require and restrict the permissible values of its
    components.

    **Attributes**

    .. attribute:: all_parts

      A sequence of known URL parts.  Defaults to the full 10-tuple of
      names in :mod:`urlparse`'s vocabulary for http-like URls.

    .. attribute:: required_parts

      A mapping of part names.  If value is True, the part is
      required.  The value may also be a sequence of strings; the
      value of the part must be present in this collection to
      validate.

      The default requires a ``scheme`` of 'http' or 'https'.

    .. attribute:: forbidden_parts

      A mapping of part names.  If value is True, the part is
      forbidden and validation fails.  The value may also be a
      sequence of strings; the value of the part must not be present
      in this collection to validate.

      The default forbids ``username`` and ``password`` parts.

    .. attribute:: urlparse

      By default the :mod:`urlparse` module, but may be replaced by
      any object that implements :func:`urlparse.urlparse` and
      :func:`urlparse.urlunparse`.

    **Messages**

    .. attribute:: bad_format

      Emitted for an unparseable URL.

    .. attribute:: required_part

      Emitted if URL is missing a part present in
      :attr:`required_parts`.

    .. attribute:: forbidden_part

      Emitted if URL contains a part present in
      :attr:`forbidden_parts`.

    """

    bad_format = N_(u'%(label)s is not a valid URL.')
    required_part = N_(u'%(label)s is not a valid URL.')
    forbidden_part = N_(u'%(label)s is not a valid URL.')

    all_parts = ('scheme', 'username', 'password', 'hostname', 'port',
                 'path', 'params', 'query', 'fragment')
    required_parts = dict(schema=('http', 'https'), hostname=True)
    forbidden_parts = dict(username=True, password=True)
    urlparse = urlparse

    def validate(self, element, state):
        url = element.value
        if url is None or not url.startswith('http'):
            return True
        parsed = self.urlparse.urlparse(url)

        for part in self.all_parts:
            try:
                value = getattr(parsed, part)
                if part == 'port':
                    value = None if value is None else str(value)
            except ValueError:
                return self.note_error(element, state, 'bad_format')
            required = self.required_parts.get(part)
            if required is True:
                if value is None:
                    return self.note_error(element, state, 'required_part')
            elif required:
                if value not in required:
                    return self.note_error(element, state, 'required_part')
            forbidden = self.forbidden_parts.get(part)
            if forbidden is True:
                if value:
                    return self.note_error(element, state, 'forbidden_part')
            elif forbidden:
                if value in forbidden:
                    return self.note_error(element, state, 'forbidden_part')
        return True


class URLCanonicalizer(Validator):
    """A URL canonicalizing validator.

    Given a valid URL, re-writes it with unwanted parts removed.  The
    default implementation drops the ``#fragment`` from the URL, if
    present.

    **Attributes**

    .. attribute:: discard_parts

      A sequence of 0 or more part names in :mod:`urlparse`'s
      vocabulary::

        'scheme', 'netloc', 'path', 'params', 'query', 'fragment'

    .. attribute:: urlparse

      By default the :mod:`urlparse` module, but may be replaced by
      any object that implements :func:`urlparse.urlparse` and
      :func:`urlparse.urlunparse`.

    **Messages**

    .. attribute:: bad_format

      Emitted for an unparseable URL.  This is impossible to hit with
      the Python's standard library implementation of urlparse.

    """

    bad_format = N_(u'%(label)s is not a valid URL.')

    discard_parts = 'fragment',
    urlparse = urlparse

    def validate(self, element, state):
        if not self.discard_parts:
            return True
        try:
            url = self.urlparse.urlparse(element.value)
        except Exception:
            return self.note_error(element, state, 'bad_format')

        url = list(url)
        for part in self.discard_parts:
            idx = _url_parts.index(part)
            current = url[idx]
            url[idx] = '' if current is not None else None

        element.value = self.urlparse.urlunparse(url)
        return True



from __future__ import division
import types
from base import N_, Validator
import flatland.util as util



class NANPnxx(Validator):
    """Integer"""

    def validate(self, element, state):
        if element.value is None:
            return False

        nxx = element.value

        if nxx < 200 or nxx in (311, 411, 511, 611, 711, 811, 911,
                                555, 990, 959, 958, 950, 700,
                                976,):
            return False

        element.u = unicode(nxx)
        return True

class NANPnpa_nxx(Validator):
    "Validates npa and nxx compound elements."

    incomplete = None

    invalid = N_(u'The %(label)s can not be verified.')

    def __init__(self, npa_element, nxx_element, errors_to=None,
                 lookup='aq', method='valid_npanxx', **kw):
        assert isinstance(npa_element, basestring)
        assert isinstance(nxx_element, basestring)
        assert isinstance(errors_to, (basestring, types.NoneType))

        Validator.__init__(self, *kw)

        self.npa = npa_element
        self.nxx = nxx_element
        self.lookup = lookup
        self.method = method

    def validate(self, element, state):
        npa = element.find(self.npa, single=True).value
        nxx = element.find(self.nxx, single=True).value

        if self.errors_to:
            err = element.find(self.errors_to)
        else:
            err = element

        if npa is None or nxx is None:
            return self.note_error(err, state, 'incomplete')

        # Will explode at run-time if state does not contain the lookup
        # tool.
        if hasattr(state, self.lookup):
            lookup = getattr(state, self.lookup)
        else:
            lookup = state[self.lookup]

        # catch exceptions here?
        valid = getattr(lookup, self.method)(npa, nxx)

        if not valid:
            return self.note_error(err, state, 'invalid')

        return True


class Luhn10(Validator):
    """Int or Long"""

    invalid = N_('The %(label)s was not entered correctly.')

    def validate(self, element, state):
        num = element.value
        if num is None:
            return self.note_error(element, state, 'invalid')

        if util.luhn10(num):
            return True

        return self.note_error(element, state, 'invalid')



from flatland import Array, String

from tests.markup._util import desired_output


def scalar_schema():
    schema = String.named(u'scalar')
    return schema(u'abc')


def multivalue_schema():
    schema = Array.named(u'multi').of(String)
    return schema([u'abc', u'xyz'])


@desired_output('xhtml', scalar_schema)
def select():
    """
<select name="scalar">
<option value="abc" selected="selected"></option>
<option value="def">DEF</option>
<option selected="selected">abc</option>
<option value="abc" selected="selected">abc</option>
</select>
"""


@select.genshi
def test_select_genshi():
    """
<select form:bind="form">
<option value="abc" form:bind="form"></option>
<option value="def">DEF</option>
<option>abc</option>
<option value="abc">abc</option>
</select>
    """


@select.markup
def test_select_markup(gen, el):
    output = []
    output += [gen.select.open(el)]
    output += [gen.option(el, value=u'abc')]
    output += [gen.option(el, value=u'def', contents=u'DEF')]
    output += [gen.option(el, contents=u'abc')]
    output += [gen.option(el, value=u'abc', contents=u'abc')]
    output += [gen.select.close()]
    return u'\n'.join(output)


@desired_output('xhtml', multivalue_schema)
def multiselect():
    """
<select name="multi" multiple="multiple">
<option value="abc" selected="selected"></option>
<option value="def">DEF</option>
<option selected="selected">xyz</option>
</select>
"""


@multiselect.genshi
def test_multiselect_genshi():
    """
<select name="multi" form:bind="form" multiple="multiple">
<option value="abc"></option>
<option value="def">DEF</option>
<option>xyz</option>
</select>
    """


@multiselect.markup
def test_multiselect_markup(gen, el):
    output = []
    output += [gen.select.open(el, multiple=u'multiple')]
    output += [gen.option(el, value=u'abc')]
    output += [gen.option(el, value=u'def', contents=u'DEF')]
    output += [gen.option(el, contents=u'xyz')]
    output += [gen.select.close()]
    return u'\n'.join(output)



"""Test suite helpers."""
import codecs
from functools import wraps
from inspect import stack
import sys

from nose.tools import eq_, assert_raises, raises


__all__ = ['asciistr', 'assert_raises', 'eq_', 'raises', 'fails',
           'requires_unicode_coercion', 'udict', 'unicode_coercion_available']

# acts like 'str', but safe to use when tests are running with
# sys.getdefaultencoding() == 'nocoercion'.
_ascii_codec = codecs.getencoder('ascii')
asciistr = lambda s: _ascii_codec(s)[0]


def fails(reason):
    """Mark a test case as expected to fail for *reason*."""
    def decorator(fn):
        @wraps(fn)
        def decorated(*args, **kw):
            try:
                fn(*args, **kw)
            except (SystemExit, KeyboardInterrupt):
                raise
            except:
                pass  # ok
            else:
                raise AssertionError("Unexpected success.")
        return decorated
    return decorator


def udict(*dictionary, **kwargs):
    "Return a dict with unicode keys. A stand-in for the dict constructor."
    kwargs = dict((k.decode('ascii'), v) for k, v in kwargs.items())
    if dictionary:
        base = dict((k.decode('ascii'), v) for k, v in dictionary[0].items())
        base.update(kwargs)
        return base
    else:
        return kwargs


def unicode_coercion_available():
    return sys.getdefaultencoding() != 'nocoercion'


def requires_unicode_coercion(fn):
    @wraps(fn)
    def decorated(*args, **kw):
        if sys.getdefaultencoding() != 'nocoercion':
            return fn(*args, **kw)
        try:
            fn(*args, **kw)
        except UnicodeError:
            # failure is success!
            return
        else:
            raise AssertionError("Test did not trigger expected UnicodeError.")
    return decorated


def _allowed_coercion(input):
    global genshis
    if isinstance(input, (int, float, long, type(None))):
        return True
    calling_file = stack()[2][1]
    if calling_file.endswith('sre_parse.py'):
        return True
    elif 'genshi' in calling_file and 'out/genshi' not in calling_file:
        # OMG slow on genshi 0.5.2
        return True
    return False


class NoCoercionCodec(codecs.Codec):

    def encode(self, input, errors='string'):
        if _allowed_coercion(input):
            return codecs.ascii_encode(input, errors)
        raise UnicodeError("encoding coercion blocked")

    def decode(self, input, errors='strict'):
        if _allowed_coercion(input):
            return codecs.ascii_decode(input, errors)
        raise UnicodeError("encoding coercion blocked")


class _IncrementalEncoder(codecs.IncrementalEncoder):

    def encode(self, input, final=False):
        if _allowed_coercion(input):
            return codecs.ascii_encode(input, self.errors)[0]
        raise UnicodeError("encoding coercion blocked")


class _IncrementalDecoder(codecs.IncrementalDecoder):

    def decode(self, input, final=False):
        if _allowed_coercion(input):
            return codecs.ascii_decode(input, self.errors)[0]
        raise UnicodeError("encoding coercion blocked")


class _StreamWriter(NoCoercionCodec, codecs.StreamWriter):
    pass


class _StreamReader(NoCoercionCodec, codecs.StreamReader):
    pass


def enable_coercion_blocker():
    registration = lambda name: codecs.CodecInfo(
        name='nocoercion',
        encode=NoCoercionCodec().encode,
        decode=NoCoercionCodec().decode,
        incrementalencoder=_IncrementalEncoder,
        incrementaldecoder=_IncrementalDecoder,
        streamwriter=_StreamWriter,
        streamreader=_StreamReader,
        )
    codecs.register(registration)
    reload(sys)
    sys.setdefaultencoding("nocoercion")



from flatland import (
    Constrained,
    Enum,
    Integer,
    )

from tests._util import eq_, assert_raises


def test_constrained_no_default_validity():
    el = Constrained(u'anything')
    assert el.value is None
    assert el.u == u'anything'


def test_constrained_instance_override():

    def make_checker(*ok_values):

        def is_valid(element, value):
            assert isinstance(element, Constrained)
            return value in ok_values
        return is_valid

    el = Constrained(valid_value=make_checker(u'a'))
    assert el.set(u'a')
    assert el.value == u'a'
    assert el.u == u'a'

    assert not el.set(u'b')
    assert el.value is None
    assert el.u == u'b'

    el = Constrained(child_type=Integer, valid_value=make_checker(1, 2))
    assert el.set(u'1')
    assert el.value == 1
    assert el.u == u'1'

    assert el.set(u'2')
    assert el.value == 2
    assert el.u == u'2'

    for invalid in u'3', u'x':
        assert not el.set(invalid)
        assert el.value == None
        assert el.u == invalid


def test_constrained_instance_contrived():
    # check that fringe types that adapt as None can used in bounds

    class CustomInteger(Integer):

        def adapt(self, value):
            try:
                return Integer.adapt(self, value)
            except:
                return None

    el = Constrained(child_type=CustomInteger,
                     valid_value=lambda e, v: v in (1, None))
    assert el.set(u'1')

    for out_of_bounds in u'2', u'3':
        assert not el.set(out_of_bounds)

    for invalid in u'x', u'':
        assert el.set(invalid)
        assert el.value is None
        assert el.u == u''


def test_default_enum():
    good_values = (u'a', u'b', u'c')
    for good_val in good_values:
        for schema in (Enum.using(valid_values=good_values),
                       Enum.valued(*good_values)):
            el = schema()
            assert el.set(good_val)
            assert el.value == good_val
            assert el.u == good_val
            assert el.validate()
            assert not el.errors

    schema = Enum.valued(*good_values)
    el = schema()
    assert not el.set(u'd')
    assert el.value is None
    assert el.u == u'd'
    # present but not converted
    assert el.validate()

    el = schema()
    assert not el.set(None)
    assert el.value is None
    assert el.u == u''
    # not present
    assert not el.validate()


def test_typed_enum():
    good_values = range(1, 4)
    schema = Enum.using(valid_values=good_values, child_type=Integer)

    for good_val in good_values:
        el = schema()
        assert el.set(unicode(str(good_val), 'ascii'))
        assert el.value == good_val
        assert el.u == unicode(str(good_val), 'ascii')
        assert not el.errors

    el = schema()
    assert not el.set(u'x')
    assert el.value is None
    assert el.u == u'x'

    el = schema()
    assert not el.set(u'5')
    assert el.value is None
    assert el.u == u'5'



from flatland.out.generic import Context, _default_context
from tests._util import assert_raises

Nothing = object()


def test_read_write_known():
    ctx = Context()

    needle = _default_context.keys()[0]
    assert needle in ctx
    assert ctx[needle] is not Nothing
    ctx[needle] = Nothing
    assert ctx[needle] is Nothing


def test_read_write_unknown():
    ctx = Context()

    needle = 'xyzzy'
    assert needle not in _default_context.keys()
    assert needle not in ctx
    assert_raises(KeyError, lambda: ctx[needle])
    assert_raises(KeyError, ctx.__setitem__, needle, Nothing)


def test_push_known():
    ctx = Context()

    needle = _default_context.keys()[0]
    assert needle in ctx
    assert ctx[needle] is not Nothing

    ctx.push(**{needle.encode('ascii'): Nothing})
    assert ctx[needle] is Nothing

    ctx.pop()
    assert ctx[needle] is not Nothing


def test_push_unknown():
    ctx = Context()

    needle = 'xyzzy'
    assert needle not in _default_context.keys()

    assert_raises(KeyError, ctx.push, **{needle: Nothing})
    assert_raises(RuntimeError, ctx.pop)


def test_update_known():
    ctx = Context()
    known = _default_context.keys()
    sentinels = [object(), object()]

    iterable = [(known[0], sentinels[0])]
    kwargs = {known[1].encode('ascii'): sentinels[1]}

    ctx.update(iterable, **kwargs)

    assert ctx[known[0]] is sentinels[0]
    assert ctx[known[1]] is sentinels[1]

    ctx = Context()
    ctx.update(iterable)
    assert ctx[known[0]] is sentinels[0]

    ctx = Context()
    ctx.update(**kwargs)
    assert ctx[known[1]] is sentinels[1]


def test_update_unknown():
    ctx = Context()
    assert u'xyzzy' not in _default_context.keys()

    assert_raises(KeyError, ctx.update, xyzzy=123)
    assert u'xyzzy' not in ctx


def test_update_bogus():
    ctx = Context()
    assert_raises(TypeError, ctx.update, [], [])


def test_minimum_repr_sanity():
    ctx = Context()
    assert repr(ctx)  # don't blow up
    assert str(ctx)   # string coercion too


def test_default_minimum_stack():
    ctx = Context()
    assert_raises(RuntimeError, ctx.pop)


def test_stack_plain_push_pop():
    ctx = Context()

    needle, initial_value = _default_context.items()[0]
    assert ctx[needle] == initial_value

    ctx.push()
    assert ctx[needle] == initial_value
    ctx[needle] = Nothing
    assert ctx[needle] is Nothing

    ctx.pop()
    assert ctx[needle] is not Nothing
    assert ctx[needle] == initial_value

    assert_raises(RuntimeError, ctx.pop)



# portions of this file are derived from SQLAlchemy
from tests._util import eq_, assert_raises
from flatland import util


def test_lazy_property():
    poison = False

    class Foo(object):

        @util.lazy_property
        def squiznart(self):
            assert not poison
            return 'abc'

    assert Foo.squiznart != 'abc'
    assert hasattr(Foo.squiznart, '__get__')

    f = Foo()
    assert 'squiznart' not in f.__dict__
    assert f.squiznart == 'abc'
    assert f.__dict__['squiznart'] == 'abc'

    poison = True
    assert f.squiznart == 'abc'

    new_foo = Foo()
    assert_raises(AssertionError, getattr, new_foo, 'squiznart')
    assert 'squiznart' not in new_foo.__dict__


def test_as_mapping():

    class Foo(object):
        clazz = 'c'

        def __init__(self):
            self.inzt = 'i'

    m = util.as_mapping(Foo)
    assert 'clazz' in m
    assert m['clazz'] == 'c'
    assert sorted(dir(Foo)) == sorted(m)
    assert_raises(KeyError, m.__getitem__, 'inzt')

    mi = util.as_mapping(Foo())
    assert 'clazz' in mi
    assert mi['clazz'] == 'c'
    assert 'inzt' in mi
    assert mi['inzt'] == 'i'
    assert sorted(dir(Foo())) == sorted(mi)


def test_luhn10():
    assert util.luhn10(0) is True
    assert util.luhn10(4100000000000001) is True
    assert util.luhn10(4100000000000009) is False


def test_to_pairs():
    to_pairs = util.to_pairs
    wanted = [('a', 1), ('b', 2)]

    assert list(to_pairs(wanted)) == wanted
    assert list(to_pairs(iter(wanted))) == wanted
    assert sorted(to_pairs(dict(wanted))) == wanted

    class Duck(object):

        def keys(self):
            return dict(wanted).keys()

        def __getitem__(self, key):
            return dict(wanted)[key]

    assert sorted(to_pairs(Duck())) == wanted


PAIRS = [('a', 1), ('b', 2), ('c', 3),
         ('d', 4), ('d', 4), ('d', 5)]


def test_keyslice_conflict():
    generator = util.keyslice_pairs((), include=[1], omit=[2])
    assert_raises(TypeError, list, generator)


def test_keyslice_pairs():
    assert list(util.keyslice_pairs(PAIRS)) == PAIRS
    assert list(util.keyslice_pairs(tuple(PAIRS))) == PAIRS
    assert list(util.keyslice_pairs(iter(PAIRS))) == PAIRS


def _keyslice_eq_(wanted, kw={}):
    got = list(util.keyslice_pairs(PAIRS, **kw))
    eq_(wanted, got)


def test_keyslice_include():
    yield _keyslice_eq_, PAIRS, dict(include=[])
    yield _keyslice_eq_, [('a', 1)], dict(include=['a'])
    yield _keyslice_eq_, [('a', 1), ('b', 2)], dict(include=['a', 'b'])
    yield _keyslice_eq_, [('d', 4), ('d', 4), ('d', 5)], dict(include=['d'])
    yield _keyslice_eq_, [('a', 1)], dict(include=['a', 'e'])


def test_keyslice_omit():
    yield _keyslice_eq_, PAIRS, dict(omit=[])
    yield _keyslice_eq_, [('a', 1), ('b', 2), ('c', 3)], dict(omit=['d'])
    yield _keyslice_eq_, [('a', 1), ('b', 2)], dict(omit=['c', 'd'])
    yield _keyslice_eq_, [('a', 1), ('b', 2)], dict(omit=['c', 'd', 'e'])
    yield _keyslice_eq_, [], dict(omit=['a', 'b', 'c', 'd'])


def test_keyslice_rename():
    wanted = PAIRS[:3] + [('Z', 4), ('Z', 4), ('Z', 5)]
    yield _keyslice_eq_, wanted, dict(rename={'d': 'Z'})
    yield _keyslice_eq_, wanted, dict(rename=[('d', 'Z')])
    yield _keyslice_eq_, wanted, dict(rename={'d': 'Z', 'e': 'Y'})

    wanted = [('d', 1), ('c', 2), ('b', 3),
              ('a', 4), ('a', 4), ('a', 5)]

    yield _keyslice_eq_, wanted, dict(rename=zip('abcddd', 'dcbaaa'))


def test_keyslice_key():
    wanted = [(int(k, 16), v) for k, v in PAIRS]

    keyfunc = lambda v: int(v, 16)
    yield _keyslice_eq_, wanted, dict(key=keyfunc)

    wanted = wanted[:3] + [(0, 4), (0, 4), (0, 5)]
    yield _keyslice_eq_, wanted, dict(key=keyfunc, rename={13: 0})


def test_keyslice_mixed():
    wanted = [('a', 1), ('X', 2)]

    yield _keyslice_eq_, wanted, dict(rename={'b': 'X'}, include=['a'])
    yield _keyslice_eq_, wanted, dict(rename={'b': 'X'}, omit=['b', 'c', 'd'])


def test_symbols():
    sym1 = util.symbol('foo')
    assert sym1.name == 'foo'
    sym2 = util.symbol('foo')

    assert sym1 is sym2
    assert sym1 == sym2

    sym3 = util.symbol('bar')
    assert sym1 is not sym3
    assert sym1 != sym3

    assert repr(sym3) == 'bar'


def test_symbol_pickle():
    import pickle
    try:
        import cPickle
    except ImportError:
        cPickle = pickle

    for mod in pickle, cPickle:
        sym1 = util.symbol('foo')
        sym2 = util.symbol('foo')

        assert sym1 is sym2

        # default
        s = pickle.dumps(sym1)
        sym3 = pickle.loads(s)

        for protocol in 0, 1, 2:
            serial = pickle.dumps(sym1)
            rt = pickle.loads(serial)
            assert rt is sym1
            assert rt is sym2



# -*- coding: utf-8; fill-column: 78 -*-
"""Class attribute-style declarative schema construction."""
from flatland.schema.base import Element
from flatland.schema.containers import Dict


__all__ = 'Form',


class _MetaForm(type):
    """Allows fields to be specified as class attribute declarations.

    Processes class declarations of the form:

      from flatland import Form, String

      class MyForm(Form):
          name = String
          favorite_color = String.using(optional=True)

    and converts them to a :attr:`~flatland.Dict.field_schema` specification
    at class construction time.  Forms may inherit from other Forms, with
    schema declarations following normal Python class property inheritance
    semantics.

    """

    def __new__(self, class_name, bases, members):
        fields = _ElementCollection()

        # collect existing fields from super classes in __mro__ order
        for base in bases:
            fields.add_unseen(getattr(base, 'field_schema', ()))

        # add / replace fields supplied in a field_schema on this class
        fields.add_and_overwrite(members.get('field_schema', ()))

        # add / replace fields declared as attributes on this class
        declared_fields = []
        for name, value in members.items():
            # TODO warn about instances found here?
            if isinstance(value, type) and issubclass(value, Element):
                if name != value.name:
                    value = value.named(name)
                declared_fields.append(value)
                del members[name]
        fields.add_and_overwrite(declared_fields)

        # the new type's field_schema is the final result of all this
        members['field_schema'] = fields.elements
        return type.__new__(self, class_name, bases, members)


class _ElementCollection(object):
    """Internal helper collection for calculating Form field inheritance."""

    def __init__(self):
        self.elements = []
        self.names = set()

    def add_unseen(self, iterable):
        """Add new items from *iterable*."""
        for field in iterable:
            if field.name in self.names:
                continue
            self.elements.append(field)
            self.names.add(field.name)

    def add_and_overwrite(self, iterable):
        """Add from *iterable*, replacing existing items of the same name."""
        for field in iterable:
            if field.name in self.names:
                for have in self.elements:
                    if have.name == field.name:
                        self.elements.remove(have)
                        break
            self.names.add(field.name)
            self.elements.append(field)


class Form(Dict):
    """A declarative collection of named fields.

    Forms behave like :class:`flatland.Dict`, but are defined with Python
    class syntax:

    .. doctest::

      >>> from flatland import Form, String
      >>> class HelloForm(Form):
      ...     hello = String
      ...     world = String
      ...

    Fields are assigned names from the declaration.  If a named schema is
    used, a renamed copy will be assigned to the Form.

    .. doctest::

      >>> class HelloForm(Form):
      ...     hello = String.named('hello')   # redundant
      ...     world = String.named('goodbye') # will be renamed 'world'
      ...
      >>> form = HelloForm()
      >>> sorted(form.keys())
      [u'hello', u'world']

    Forms may embed other container fields and other forms:

    .. doctest::

      >>> from flatland import List
      >>> class BigForm(Form):
      ...     main_hello = HelloForm
      ...     alt_hello = List.of(String.named('alt_name'),
      ...                         HelloForm.named('alt_hello'))
      ...

    This would create a form with one ``HelloForm`` embedded as
    ``main_hello``, and a list of zero or more dicts, each containing an
    ``alt_name`` and another ``HelloForm`` named ``alt_hello``.

    Forms may inherit from other Forms or Dicts.  Field declared in a subclass
    will override those of a superclass.  Multiple inheritance is supported.

    The special behavior of ``Form`` is limited to class construction time
    only.  After construction, the ``Form`` acts exactly like a
    :class:`~flatland.Dict`.  In particular, fields declared in class
    attribute style do **not** remain class attributes.  They are removed from
    the class dictionary and placed in the
    :attr:`~flatland.Dict.field_schema`:

    .. doctest::

      >>> hasattr(HelloForm, 'hello')
      False
      >>> sorted([field.name for field in HelloForm.field_schema])
      [u'hello', u'world']

    The order of ``field_schema`` after construction is undefined.

    """

    __metaclass__ = _MetaForm

    # TODO:
    #   some kind of validator merging helper?  or punt?



from functools import update_wrapper
import operator

from flatland.exc import AdaptationError
from flatland.util import Unspecified, threading
from .containers import Array, Mapping
from .scalars import Date, Integer, Scalar, String


class _MetaCompound(type):
    """Adds a class-level initialization hook """

    _lock = threading.RLock()

    def __new__(self, name, bases, members):
        members['_compound_prepared'] = False
        if '__compound_init__' in members:
            members['__compound_init__'] = \
              _wrap_compound_init(members['__compound_init__'])
        return type.__new__(self, name, bases, members)

    def __call__(cls, value=Unspecified, **kw):
        """Run __compound_init__ on first instance construction."""

        # Find **kw that would override existing class properties and
        # remove them from kw.
        overrides = {}
        for key in kw.keys():
            if hasattr(cls, key):
                overrides[key] = kw.pop(key)

        if overrides:
            # If there are overrides, construct a subtype on the fly
            # so that __compound_init__ has a chance to run again with
            # this new configuration.
            cls = cls.using(**overrides)
            cls.__compound_init__()
        elif not cls.__dict__.get('_compound_prepared'):
            # If this class hasn't been prepared yet, prepare it.
            lock = cls._lock
            lock.acquire()
            try:
                if not cls.__dict__.get('_compound_prepared'):
                    cls.__compound_init__()
            finally:
                lock.release()

        # Finally, implement type.__call__, invoking __init__ with the
        # members of kw that were not class overrides.
        self = cls.__new__(cls, value, **kw)
        if self.__class__ is cls:
            if value is Unspecified:
                self.__init__(**kw)
            else:
                self.__init__(value, **kw)
        return self


def _wrap_compound_init(fn):
    """Decorate __compound_init__ with a status setter & classmethod."""
    if isinstance(fn, classmethod):
        fn = fn.__get__(str).im_func  # type doesn't matter here
    def __compound_init__(cls):
        res = fn(cls)
        cls._compound_prepared = True
        return res
    update_wrapper(__compound_init__, fn)
    return classmethod(__compound_init__)


class Compound(Mapping, Scalar):
    """A mapping container that acts like a scalar value.

    Compound fields are dictionary-like fields that can assemble a
    :attr:`.u` and :attr:`.value` from their children, and can
    decompose a structured value passed to a :meth:`.set` into values
    for its children.

    A simple example is a logical calendar date field composed of 3
    separate Integer component fields, year, month and day.  The
    Compound can wrap the 3 parts up into a single logical field that
    handles :class:`datetime.date` values.  Set a ``date`` on the
    logical field and its component fields will be set with year,
    month and day; alter the int value of the year component field and
    the logical field updates the ``date`` to match.

    :class:`Compound` is an abstract class.  Subclasses must implement
    :meth:`compose` and :meth:`explode`.

    Composites run validation after their children.

    """

    __metaclass__ = _MetaCompound

    def __compound_init__(cls):
        """TODO: doc

        Gist: runs *once* per class, at the time the first element is
        constructed.  You can run it by hand if you want to finalize
        the construction (see TODO above).

        Changing class params on instance construction will cause this
        to run again.

        """

    def compose(self):
        """Return a unicode, native tuple built from children's state.

        :returns: a 2-tuple of unicode representation, native value.
           These correspond to the :meth:`Scalar.serialize_element`
           and :meth:`Scalar.adapt_element` methods of :class:`Scalar`
           objects.

        For example, a compound date field may return a '-' delimited
        string of year, month and day digits and a
        :class:`datetime.date`.

        """
        raise NotImplementedError()

    def explode(self, value):
        """Given a compound value, assign values to children.

        :param value: a value to be adapted and exploded

        For example, a compound date field may read attributes from a
        :class:`datetime.date` value and :meth:`.set()` them on child
        fields.

        The decision to perform type checking on *value* is completely
        up to you and you may find you want different rules for
        different compound types.

        """
        raise NotImplementedError()

    def serialize(self, value):
        raise TypeError("Not implemented for Compound types.")

    def u(self):
        uni, value = self.compose()
        return uni

    def set_u(self, value):
        self.explode(value)

    u = property(u, set_u)
    del set_u

    def value(self):
        uni, value = self.compose()
        return value

    def set_value(self, value):
        self.explode(value)

    value = property(value, set_value)
    del set_value

    def set(self, value):
        try:
            # TODO: historically explode() did not need to have a return value
            # but it would be nice to return it form set() as below.
            res = self.explode(value)
            return True if res is None else res
        except (SystemExit, KeyboardInterrupt, NotImplementedError):
            raise
        except Exception:
            # not wild about quashing here, but set() doesn't allow
            # adaptation exceptions to bubble up.
            return False

    def _set_flat(self, pairs, sep):
        Mapping._set_flat(self, pairs, sep)

    def __repr__(self):
        try:
            return Scalar.__repr__(self)
        except Exception, exc:
            return '<%s %r; value raised %s>' % (
                type(self).__name__, self.name, type(exc).__name__)

    @property
    def is_empty(self):
        """True if all subfields are empty."""
        return reduce(operator.and_, (c.is_empty for c in self.children))


class DateYYYYMMDD(Compound, Date):

    @classmethod
    def __compound_init__(cls):
        assert len(cls.field_schema) < 4
        if len(cls.field_schema) == 3:
            return

        fields = list(cls.field_schema)
        optional = cls.optional

        if len(fields) == 0:
            fields.append(Integer.named(u'year').using(format=u'%04i',
                                                       optional=optional))
        if len(fields) == 1:
            fields.append(Integer.named(u'month').using(format=u'%02i',
                                                        optional=optional))
        if len(fields) == 2:
            fields.append(Integer.named(u'day').using(format=u'%02i',
                                                      optional=optional))

        cls.field_schema = fields

    #if any(not field.name for field in fields):
    #    raise TypeError("Child fields of %s %r must be named." %
    #                    type(self).__name__, name)

    def compose(self):
        try:
            data = dict([(label, self[child_schema.name].value)
                         for label, child_schema
                         in zip(self.used, self.field_schema)])
            as_str = self.format % data
            value = Date.adapt(self, as_str)

            return as_str, value
        except (AdaptationError, TypeError):
            return u'', None

    def explode(self, value):
        try:
            value = Date.adapt(self, value)

            for attrib, child_schema in zip(self.used, self.field_schema):
                self[child_schema.name].set(
                    getattr(value, attrib.encode('ascii')))
        except (AdaptationError, TypeError):
            for child_schema in self.field_schema:
                self[child_schema.name].set(None)


class JoinedString(Array, String):
    """A sequence container that acts like a compounded string such as CSV.

    Marshals child element values to and from a single string:

    .. doctest::

      >>> from flatland import JoinedString
      >>> el = JoinedString(['x', 'y', 'z'])
      >>> el.value
      u'x,y,z'
      >>> el2 = JoinedString('foo,bar')
      >>> el2[1].value
      u'bar'
      >>> el2.value
      u'foo,bar'

    Only the joined representation is considered when flattening or restoring
    with :meth:`set_flat`.  JoinedStrings run validation after their children.

    """
    #: The string used to join children's :attr:`u` representations.  Will
    #: also be used to split incoming strings, unless :attr:`separator_regex`
    #: is also defined.
    separator = u','

    #: Optional, a regular expression, used preferentially to split an
    #: incoming separated value into components.  Used in combination with
    #: :attr:`separator`, a permissive parsing policy can be combined with
    #: a normalized representation, e.g.:
    #:
    #: .. doctest::
    #:
    #:   >>> import re
    #:   >>> schema = JoinedString.using(separator=', ',
    #:   ...                             separator_regex=re.compile('\s*,\s*'))
    #:   ...
    #:   >>> schema('a  ,  b,c,d').value
    #:   u'a, b, c, d'
    separator_regex = None

    #: The default child type is :class:`~flatland.schema.scalars.String`,
    #: but can be customized with
    #: :class:`~flatland.schema.scalars.Integer` or any other type.
    member_schema = String

    flattenable = True
    children_flattenable = False

    def set(self, value):
        if isinstance(value, (list, tuple)):
            values = value
        elif not isinstance(value, basestring):
            values = list(value)
        elif self.separator_regex:
            # a basestring, regexp separator
            values = self.separator_regex.split(value)
        else:
            # a basestring, static separator
            values = value.split(self.separator)

        del self[:]
        prune = self.prune_empty
        success = []
        for value in values:
            if prune and not value:
                continue
            child = self.member_schema()
            success.append(child.set(value))
            self.append(child)
        return all(success)

    def _set_flat(self, pairs, sep):
        return Scalar._set_flat(self, pairs, sep)

    @property
    def value(self):
        """A read-only :attr:`separator`-joined string of child values."""
        return self.separator.join(child.u for child in self)

    @property
    def u(self):
        """A read-only :attr:`separator`-joined string of child values."""
        return self.value



from tests.markup._util import desired_output


def simple_schema():
    from flatland import Form, String

    class SmallForm(Form):
        valued = String
        empty = String

    return SmallForm({u'valued': u'val'})

###


@desired_output('html', simple_schema)
def input_value_html():
    """<input name="valued" value="val">"""


@input_value_html.genshi
def test_input_value_html_genshi():
    """<input form:bind="form.valued"/>"""


@input_value_html.markup
def test_input_value_html_markup(gen, el):
    return gen.input(el['valued'])

###


@desired_output('xhtml', simple_schema)
def input_value_xhtml():
    """<input name="valued" value="val" />"""


@input_value_xhtml.genshi
def test_input_value_xhtml_genshi():
    """<input form:bind="form.valued"/>"""


@input_value_xhtml.markup
def test_input_value_xhtml_markup(gen, el):
    return gen.input(el['valued'])

###


@desired_output('xhtml', simple_schema)
def textarea_value():
    """<textarea name="valued">val</textarea>"""


@textarea_value.genshi
def test_textarea_value_genshi():
    """<textarea form:bind="form.valued"/>"""


@textarea_value.markup
def test_textarea_value_markup(gen, el):
    return gen.textarea(el['valued'])

###


@desired_output('xhtml', simple_schema)
def textarea_empty_value():
    """<textarea name="empty"></textarea>"""


@textarea_empty_value.genshi
def test_textarea_empty_value_genshi():
    """<textarea form:bind="form.empty"/>"""


@textarea_empty_value.markup
def test_textarea_empty_value_markup(gen, el):
    return gen.textarea(el['empty'])

###


@desired_output('xhtml', simple_schema)
def textarea_explicit_value():
    """<textarea name="valued">override</textarea>"""


@textarea_explicit_value.genshi
def test_textarea_explicit_value_genshi():
    """<textarea form:bind="form.valued">override</textarea>"""


@textarea_explicit_value.markup
def test_textarea_explicit_value_markup(gen, el):
    return gen.textarea(el['valued'], contents='override')

###


@desired_output('html', simple_schema)
def label_empty_html():
    """<label></label>"""


@label_empty_html.genshi
def test_label_empty_html_genshi():
    """<label form:bind="form.valued"/>"""


@label_empty_html.markup
def test_label_empty_html_markup(gen, el):
    return gen.label(el['valued'])



from __future__ import absolute_import
from collections import deque

from genshi.core import Namespace, QName, END, START, TEXT
from genshi.template.base import (
    DirectiveFactory,
    EXPR,
    SUB,
    TemplateSyntaxError,
    _eval_expr,
    )
from genshi.template.eval import Expression
from genshi.template.directives import Directive
from genshi.template.interpolation import interpolate


from flatland.out.generic import _unpack, transform, Context


__all__ = ('setup',)

NS = Namespace(u'http://ns.discorporate.us/flatland/genshi')

_static_attribute_order = [u'type', u'name', u'value']

_to_context = {}
for key in (u'auto-name', u'auto-value', u'auto-domid', u'auto-for',
            u'auto-tabindex', u'auto-filter', u'domid-format'):
    _to_context[key] = key.replace(u'-', u'_')

_bind_qname = NS.bind


def setup(template):
    """Register the flatland directives with a template.

    :param template: a `Template` instance
    """

    if not hasattr(template, 'add_directives'):
        raise RuntimeError("%s.setup requires Genshi 0.6 or higher." % (
            __name__,))
    template.add_directives(NS, FlatlandElements())


class EvaluatedLast(Directive):
    __slots__ = ()

    def __call__(self, stream, directives, ctxt, **vars):
        local, foreign = [], []
        for d in directives:
            if isinstance(d, EvaluatedLast):
                local.append(d)
            else:
                foreign.append(d)
        if foreign:
            foreign.append(self)
            foreign.extend(local)
            return foreign[0](stream, foreign[1:], ctxt, **vars)
        return self.process(stream, local, ctxt, vars)

    def process(self, stream, directives, ctxt, vars):
        raise NotImplementedError  # pragma: nocover


class TagOnly(EvaluatedLast):
    _name = None
    __slots__ = ('attributes',)

    @classmethod
    def attach(cls, template, stream, value, namespaces, pos):
        if type(value) is not dict:
            raise TemplateSyntaxError(
                "The %s directive must be an element" % cls._name,
                template.filepath, *pos[1:])
        return super(TagOnly, cls).attach(
            template, stream, value, namespaces, pos)

    def __init__(self, value, template=None, namespaces=None,
                 lineno=-1, offset=-1):
        Directive.__init__(self, None, template, namespaces, lineno, offset)
        self.attributes = value


class AttributeOnly(EvaluatedLast):
    _name = None
    __slots__ = ()

    @classmethod
    def attach(cls, template, stream, value, namespaces, pos):
        if type(value) is dict:
            raise TemplateSyntaxError(
                ("The %s directive may only be used as a "
                 "tag attribute" % cls._name),
                template.filepath, *pos[1:])
        return super(AttributeOnly, cls).attach(
            template, stream, value, namespaces, pos)


class ControlAttribute(AttributeOnly):
    __slots__ = ('raw_value')

    def __init__(self, value, template=None, namespaces=None,
                 lineno=-1, offset=-1):
        Directive.__init__(self, None, template, namespaces, lineno, offset)

        # allow interpolation inside control attributes
        raw_value = list(interpolate(value, lineno=lineno, offset=offset))
        if all(kind is TEXT for (kind, _, _) in raw_value):
            self.raw_value = u''.join(event[1] for event in raw_value)
        else:
            self.raw_value = raw_value

    def process(self, stream, directives, ctxt, vars):
        # unbound transformation.
        if not directives:
            directives = [self]
        else:
            directives = [self] + directives
        return _rewrite_stream(stream, directives, ctxt, vars, None)

    def inject(self, mapping, ctxt, vars):
        """Inject the translated key and interpolated value into *mapping*."""
        raw = self.raw_value
        if raw.__class__ is unicode:
            final_value = raw
        else:
            parts = []
            for kind, value, pos in raw:
                if kind is TEXT:
                    parts.append(value)
                else:
                    value = _eval_expr(value, ctxt, vars)
                    parts.append(unicode(value))
            final_value = u''.join(parts)
        mapping[_to_context.get(self._name, self._name)] = final_value


class AutoName(ControlAttribute):
    _name = 'auto-name'
    __slots__ = ()


class AutoValue(ControlAttribute):
    _name = 'auto-value'
    __slots__ = ()


class AutoDomID(ControlAttribute):
    _name = 'auto-domid'
    __slots__ = ()


class AutoFor(ControlAttribute):
    _name = 'auto-for'
    __slots__ = ()


class AutoTabindex(ControlAttribute):
    _name = 'auto-tabindex'
    __slots__ = ()


class AutoFilter(ControlAttribute):
    _name = 'auto-filter'
    __slots__ = ()


class Binding(AttributeOnly):
    _name = 'bind'
    __slots__ = ('bind',)

    def __init__(self, attributes, template=None, namespaces=None,
                 lineno=-1, offset=-1, bind=None):
        AttributeOnly.__init__(self, attributes, template, namespaces,
                               lineno, offset)
        self.bind = bind

    def process(self, stream, directives, ctxt, vars):
        if self.bind is not None:
            bind = self.bind
        elif self.expr is None:
            bind = None
        else:
            bind = _eval_expr(self.expr, ctxt, vars)
        return _rewrite_stream(stream, directives, ctxt, vars, bind)


class RenderContextManipulator(TagOnly):
    __slots__ = ()

    def __init__(self, attributes, template=None, namespaces=None,
                 lineno=-1, offset=-1):
        transformed = {}
        for key, value in attributes.items():
            key = _to_context.get(key, key)
            if key == u'tabindex':
                value = int(value)
            transformed[key] = value
        TagOnly.__init__(self, transformed, template, namespaces,
                         lineno, offset)


class With(RenderContextManipulator):
    _name = 'with'
    __slots__ = ()

    def process(self, stream, directives, ctxt, vars):
        try:
            render_context = ctxt['flatland_render_context']
        except KeyError:
            ctxt['flatland_render_context'] = render_context = Context()

        if 'filters' not in self.attributes:
            attrs = self.attributes
        else:
            attrs = self.attributes.copy()
            attrs['filters'] = _eval_expr(Expression(attrs['filters']),
                                          ctxt, vars)

        render_context.push()
        render_context.update(attrs)
        assert not directives
        for event in stream:
            yield event
        render_context.pop()


class Set(RenderContextManipulator):
    _name = 'set'
    __slots__ = ()

    def process(self, stream, directives, ctxt, vars):
        try:
            render_context = ctxt['flatland_render_context']
        except KeyError:
            ctxt['flatland_render_context'] = render_context = Context()
        render_context.update(self.attributes)
        assert not directives
        return stream


class FlatlandElements(DirectiveFactory):

    NAMESPACE = NS

    directives = [
        ('with', With),
        ('set', Set),
        ('bind', Binding),
        ('auto-name', AutoName),
        ('auto-value', AutoValue),
        ('auto-domid', AutoDomID),
        ('auto-for', AutoFor),
        ('auto-tabindex', AutoTabindex),
        ('auto-filter', AutoFilter),
        ]


def _rewrite_stream(stream, directives, ctxt, vars, bind):
    stream = list(stream)
    mutable_attrs = {}

    for control_attribute in directives:
        control_attribute.inject(mutable_attrs, ctxt, vars)

    kind, (tagname, attrs), pos = stream[0]
    if len(stream) == 2:
        contents = None
    else:
        contents = _simplify_stream(stream[1:-1], ctxt, vars)

    existing_attributes = {}
    for qname, value in attrs:
        if qname.namespace is None:
            if not isinstance(value, unicode):
                value = _simplify_stream(value, ctxt, vars)
                attrs |= ((qname, value),)
            existing_attributes[qname.localname] = qname
            mutable_attrs[qname.localname] = value

    try:
        render_context = ctxt['flatland_render_context']
    except KeyError:
        ctxt['flatland_render_context'] = render_context = Context()

    new_contents = transform(tagname.localname, mutable_attrs, contents,
                             render_context, bind)

    if new_contents is None:
        new_contents = ()
    elif isinstance(new_contents, unicode):
        new_contents = [(TEXT, new_contents, (None, -1, -1))]

    pairs = sorted(mutable_attrs.iteritems(), key=_attribute_sort_key)
    for attribute_name, value in pairs:
        if attribute_name in existing_attributes:
            qname = existing_attributes.pop(attribute_name)
        else:
            qname = QName(attribute_name)
        attrs |= ((qname, value),)
    for qname in existing_attributes.values():
        attrs -= qname

    stream[0] = (kind, (tagname, attrs), pos)
    if new_contents and tagname.localname == u'select' and bind is not None:
        if tagname.namespace:
            sub_tag = Namespace(tagname.namespace).option
        else:  # pragma: nocover
            sub_tag = QName('option')
        new_contents = _bind_unbound_tags(new_contents, sub_tag, bind)
    if new_contents:
        stream[1:-1] = new_contents
    return iter(stream)


def _attribute_sort_key(item):
    try:
        return (0, _static_attribute_order.index(item[0]))
    except ValueError:
        return (1, item[0])


def _bind_unbound_tags(stream, qname, bind):
    stream = deque(stream)
    while stream:
        kind, data, pos = stream.popleft()
        if kind is SUB:
            directives, substream = data
            for d in directives:  # pragma: nocover   (coverage bug :()
                if isinstance(d, Binding):
                    break
            else:
                substream = list(_bind_unbound_tags(substream, qname, bind))
            yield kind, (directives, substream), pos
        elif kind is START:
            if data[0] != qname or qname in data[1]:
                yield kind, data, pos
                continue
            head = kind, data, pos
            substream = []
            stack = 1
            while stack:
                event = stream.popleft()
                substream.append(event)
                if event[0] is START and event[1][0] == qname:
                    stack += 1
                elif event[0] is END and event[1] == qname:
                    stack -= 1
            substream = [head] + list(
                _bind_unbound_tags(substream, qname, bind))
            # attaching the directive is sufficient; don't need to fabricate
            # a form:bind="" attribute
            yield SUB, ([Binding(u'', bind=bind)], substream), pos
        else:
            yield kind, data, pos


def _simplify_stream(stream, ctxt, vars):
    # consumes stream, send a list
    parts = []
    for idx, (kind, data, pos) in enumerate(stream):
        if kind is TEXT:
            parts.append(data)
        elif kind is EXPR:
            value = _eval_expr(data, ctxt, vars)
            if hasattr(value, '__html__'):
                value = _unpack(value)
            if hasattr(value, '__next__') or hasattr(value, 'next'):
                while hasattr(value, '__next__') or hasattr(value, 'next'):
                    value = list(value)
                    value = _simplify_stream(value, ctxt, vars)
                if not isinstance(value, unicode):
                    stream[idx:idx + 1] = value
                else:
                    stream[idx] = (TEXT, value, pos)
            elif not isinstance(value, unicode):
                value = unicode(value)
            parts.append(value)
        else:
            return stream
    return u''.join(parts)



#! -*- coding: utf-8 -*-
from flatland import Array, Boolean, Integer
from flatland.out import generic
from flatland.out.generic import Context


Unspecified = object()
Unique = object()
schema = Integer.named(u'number')
boolean_schema = Boolean.named(u'bool')
partial_anon_schema = Array.named(u'array').of(Integer)
full_anon_schema = Array.of(Integer)


def assert_bound_transform(fn, tagname, given, expected, **kw):
    return assert_transform(fn, tagname, given, expected, **kw)


def assert_unbound_transform(fn, tagname, given, expected, **kw):
    kw['bind'] = None
    return assert_transform(fn, tagname, given, expected, **kw)


def assert_transform(fn, tagname, given, expected,
                     context=Unspecified,
                     bind=Unspecified,
                     contents=Unspecified,
                     expected_contents=Unspecified):
    if context is Unspecified:
        context = Context()
    if bind is Unspecified:
        bind = schema(123)

    got = given.copy()
    got_contents = fn(tagname, got, contents, context, bind)
    assert got == expected
    assert got_contents == expected_contents

    types = zip(sorted(got.keys()), sorted(expected.keys()))
    for got_key, expected_key in types:
        assert type(got_key) is type(expected_key)


class _TestAttributeTransform(object):
    attribute = NotImplemented
    applied_value = NotImplemented
    control_attribute = NotImplemented
    tagname = NotImplemented
    skip_tags = ()
    operates_on_unbound = False

    bind_factory = lambda: schema(123)
    transform = NotImplemented
    base_context = ()

    def assert_transform(self, given, expected,
                         context=None,
                         tagname=None,
                         bind=Unspecified):
        if hasattr(self.transform, 'im_func'):
            transform = self.transform.im_func
        else:
            transform = self.transform
        if context is None:
            context = self._context()
        if tagname is None:
            tagname = self.tagname
        if bind is Unspecified:
            if hasattr(self.bind_factory, 'im_func'):
                bind = self.bind_factory.im_func()
            else:
                bind = self.bind_factory()

        assert_transform(transform, tagname, given, expected, context, bind)

    def _context(self, overrides=()):
        context = Context()
        context.push()
        context.update(self.base_context)
        context.update(overrides)
        return context

    def test_known_tag(self):
        given = {}
        if generic._default_context[self.control_attribute] is True:
            expected = {self.attribute: self.applied_value}
        elif generic._default_context[self.control_attribute] is False:
            expected = {}
        else:
            raise AssertionError

        for tagname in generic._auto_tags[self.attribute]:
            if tagname in self.skip_tags:
                continue
            yield self.assert_transform, given, expected, None, tagname

    def test_unknown_tag(self):
        given = {}
        expected = {}
        self.assert_transform(given, expected, tagname=u'xyzzy')

    def test_unknown_tag_existing_value_unchanged(self):
        given = {self.attribute: u'existing'}
        expected = {self.attribute: u'existing'}
        self.assert_transform(given, expected, tagname=u'xyzzy')

    def test_existing_value_unchanged(self):
        given = {self.attribute: u'local'}
        expected = {self.attribute: u'local'}
        self.assert_transform(given, expected)

    def test_existing_value_force_changed(self):
        given = {self.attribute: u'local', self.control_attribute: u'on'}
        expected = {self.attribute: self.applied_value}
        self.assert_transform(given, expected)

    def test_context_off(self):
        given = {}
        expected = {}
        context = self._context({self.control_attribute: False})
        self.assert_transform(given, expected, context=context)

    def test_context_on(self):
        given = {}
        expected = {self.attribute: self.applied_value}
        context = self._context({self.control_attribute: True})
        self.assert_transform(given, expected, context=context)

    def test_context_on_unknown_tag(self):
        given = {}
        expected = {}
        context = self._context({self.control_attribute: True})
        self.assert_transform(given, expected,
                              context=context, tagname=u'xyzzy')

    def test_context_auto(self):
        given = {}
        default = generic._default_context[self.control_attribute]
        if default is True:
            expected = {self.attribute: self.applied_value}
        elif default is False:
            expected = {}
        else:
            raise AssertionError()
        context = self._context({self.control_attribute: u'auto'})
        self.assert_transform(given, expected, context=context)

    def test_local_off(self):
        given = {self.control_attribute: False}
        expected = {}
        self.assert_transform(given, expected)

    def test_local_on(self):
        given = {self.control_attribute: True}
        expected = {self.attribute: self.applied_value}
        context = self._context({self.control_attribute: False})
        self.assert_transform(given, expected, context=context)

    def test_local_auto(self):
        given = {self.control_attribute: u'auto'}
        expected = {self.attribute: self.applied_value}
        context = self._context({self.control_attribute: True})
        self.assert_transform(given, expected, context=context)

        expected = {}
        context = self._context({self.control_attribute: False})
        self.assert_transform(given, expected, context=context)

    def test_local_on_unbound(self):
        # some can't be forced unless a bind is also present
        given = {self.control_attribute: True}
        if self.operates_on_unbound:
            expected = {self.attribute: self.applied_value}
        else:
            expected = {}
        self.assert_transform(given, expected, bind=None)


class TestName(_TestAttributeTransform):
    attribute = u'name'
    control_attribute = u'auto_name'
    applied_value = u'number'
    transform = generic.transform_name
    tagname = u'input'


# basic smoke test for context issues only
class TestTextboxValue(_TestAttributeTransform):
    attribute = u'value'
    control_attribute = u'auto_value'
    applied_value = u'123'
    transform = generic.transform_value
    tagname = u'input'
    skip_tags = ['textarea', 'option']


class TestButtonValue(_TestAttributeTransform):
    attribute = u'value'
    control_attribute = u'auto_value'
    applied_value = u'123'
    transform = generic.transform_value
    tagname = u'button'
    skip_tags = ['textarea', 'option']


class TestDomID(_TestAttributeTransform):
    attribute = u'id'
    control_attribute = u'auto_domid'
    applied_value = u'f_number'
    transform = generic.transform_domid
    tagname = u'input'


class TestFor(_TestAttributeTransform):
    attribute = u'for'
    control_attribute = u'auto_for'
    applied_value = u'f_number'
    transform = generic.transform_for
    tagname = u'label'


class TestTabindex(_TestAttributeTransform):
    attribute = u'tabindex'
    control_attribute = u'auto_tabindex'
    applied_value = u'5'
    operates_on_unbound = True
    transform = generic.transform_tabindex
    tagname = u'input'
    base_context = {u'tabindex': 5}


def test_name_anonymous_bind():
    bind = partial_anon_schema([123])[0]
    given = {}
    expected = {u'name': u'array'}
    assert_transform(generic.transform_name,
                     u'input', given, expected, bind=bind)

    bind = full_anon_schema([123])[0]
    given = {}
    expected = {}
    assert_transform(generic.transform_name,
                     u'input', given, expected, bind=bind)


def test_contents_textarea():
    given = None
    expected = u'123'
    assert_bound_transform(generic.transform_value,
                           u'textarea', {}, {},
                           contents=given, expected_contents=expected)

    for given in u'existing_value', u'', generic.Markup('xyzzy'), []:
        expected = given
        assert_bound_transform(generic.transform_value,
                               u'textarea', {}, {},
                               contents=given, expected_contents=expected)


def test_value_checkable():
    # check Scalar(..) and Array.of(..)
    for bind in schema(123), partial_anon_schema([123, 456]):
        for type in u'checkbox', u'radio':
            given = {
                u'type': type,
                u'value': u'123',
                }
            expected = {
                u'type': type,
                u'value': u'123',
                u'checked': u'checked',
                }
            assert_transform(generic.transform_value,
                             u'input', given, expected, bind=bind)

            given = {
                u'type': type,
                u'value': u'999',
                }
            expected = {
                u'type': type,
                u'value': u'999',
                }
            assert_transform(generic.transform_value,
                             u'input', given, expected, bind=bind)

            given = {
                u'type': type,
                u'value': u'999',
                u'checked': u'checked',
                }
            expected = {
                u'type': type,
                u'value': u'999',
                }
            assert_transform(generic.transform_value,
                             u'input', given, expected, bind=bind)

            given = {
                u'type': type,
                u'value': u'999',
                u'checked': u'',
                }
            expected = {
                u'type': type,
                u'value': u'999',
                }
            assert_transform(generic.transform_value,
                             u'input', given, expected, bind=bind)


def test_value_checkbox_boolean():
    bind = boolean_schema(True)
    given = {
        u'type': u'checkbox',
        }
    expected = {
        u'type': u'checkbox',
        u'value': u'1',
        u'checked': u'checked',
        }
    assert_transform(generic.transform_value,
                     u'input', given, expected, bind=bind)

    bind = boolean_schema(False)
    given = {
        u'type': u'checkbox',
        }
    expected = {
        u'type': u'checkbox',
        u'value': u'1',
        }
    assert_transform(generic.transform_value,
                     u'input', given, expected, bind=bind)

    bind = boolean_schema(True)
    given = {
        u'type': u'checkbox',
        u'value': u'2',
        }
    expected = {
        u'type': u'checkbox',
        u'value': u'2',
        }
    assert_transform(generic.transform_value,
                     u'input', given, expected, bind=bind)


def test_value_omitted():
    bind = schema(123)

    for type in u'password', u'file', u'image':
        given = {
            u'type': type,
            }
        expected = {
            u'type': type,
            }
        assert_transform(generic.transform_value,
                         u'input', given, expected, bind=bind)
        given = {
            u'type': type,
            u'value': u'999',
            }
        expected = {
            u'type': type,
            u'value': u'999',
            }
        assert_transform(generic.transform_value,
                         u'input', given, expected, bind=bind)
        given = {
            u'type': type,
            u'auto_value': True,
            }
        expected = {
            u'type': type,
            u'value': u'123',
            }
        assert_transform(generic.transform_value,
                         u'input', given, expected, bind=bind)

        given = {
            u'type': type,
            u'auto_value': True,
            u'value': u'999',
            }
        expected = {
            u'type': type,
            u'value': u'123',
            }
        assert_transform(generic.transform_value,
                         u'input', given, expected, bind=bind)


def test_value_option():
 # check Scalar(..) and Array.of(..)
    for bind in schema(123), partial_anon_schema([123, 456]):
        given = {
            u'value': u'123',
            }
        expected = {
            u'value': u'123',
            u'selected': u'selected',
            }
        assert_transform(generic.transform_value,
                         u'option', given, expected, bind=bind)

        given = {
            u'value': u'999',
            }
        expected = {
            u'value': u'999',
            }
        assert_transform(generic.transform_value,
                         u'option', given, expected, bind=bind)

        contents = expected_contents = u'123'
        given = {
            }
        expected = {
            u'selected': u'selected',
            }
        assert_transform(generic.transform_value,
                         u'option', given, expected, bind=bind,
                         contents=contents,
                         expected_contents=expected_contents)

        contents = expected_contents = u'    123     '
        given = {
            }
        expected = {
            u'selected': u'selected',
            }
        assert_transform(generic.transform_value,
                         u'option', given, expected, bind=bind,
                         contents=contents,
                         expected_contents=expected_contents)

        contents = expected_contents = u'999'
        given = {
            }
        expected = {
            }
        assert_transform(generic.transform_value,
                         u'option', given, expected, bind=bind,
                         contents=contents,
                         expected_contents=expected_contents)

        contents = expected_contents = None
        given = {
            }
        expected = {
            }
        assert_transform(generic.transform_value,
                         u'option', given, expected, bind=bind,
                         contents=contents,
                         expected_contents=expected_contents)

        # This matches value = contents via a sentinel object
        given = {
            }
        expected = {
            }
        assert_transform(generic.transform_value,
                         u'option', given, expected, bind=bind)

        contents = expected_contents = u'123'
        given = {
            u'value': u'999',
            }
        expected = {
            u'value': u'999',
            }
        assert_transform(generic.transform_value,
                         u'option', given, expected, bind=bind,
                         contents=contents,
                         expected_contents=expected_contents)

        given = {
            u'value': u'999',
            u'selected': u'selected',
            }
        expected = {
            u'value': u'999',
            }
        assert_transform(generic.transform_value,
                         u'option', given, expected, bind=bind)

        given = {
            u'value': u'999',
            u'selected': u'',
            }
        expected = {
            u'value': u'999',
            }
        assert_transform(generic.transform_value,
                         u'option', given, expected, bind=bind)


def test_domid_checkable_array():
    bind = partial_anon_schema([123, 456])
    context = Context()
    context[u'auto_domid'] = True

    assert_ = lambda given, expected: assert_transform(
        generic.transform_domid, u'input', given, expected,
        context=context, bind=bind)

    for type in 'radio', 'checkbox':
        given = {
            u'type': u'radio',
            u'value': u'xxx',
            }
        expected = {
            u'type': u'radio',
            u'value': u'xxx',
            u'id': u'f_array_xxx',
            }
        assert_(given, expected)

        given = {
            u'type': u'radio',
            }
        expected = {
            u'type': u'radio',
            u'id': u'f_array',
            }
        assert_(given, expected)

        given = {
            u'type': u'radio',
            u'value': u'Ḑộộḏ!',
            }
        expected = {
            u'type': u'radio',
            u'value': u'Ḑộộḏ!',
            u'id': u'f_array',
            }
        assert_(given, expected)

        given = {
            u'type': u'radio',
            u'value': u'Ḑộộḏ! yo',
            }
        expected = {
            u'type': u'radio',
            u'value': u'Ḑộộḏ! yo',
            u'id': u'f_array_yo',
            }
        assert_(given, expected)


def test_domid_checkable_scalar():
    context = Context()
    context[u'auto_domid'] = True

    for type in 'radio', 'checkbox':
        given = {
            u'type': u'radio',
            u'value': u'xxx',
            }
        expected = {
            u'type': u'radio',
            u'value': u'xxx',
            u'id': u'f_number_xxx',
            }
        assert_bound_transform(generic.transform_domid,
                               u'input', given, expected, context=context)


def test_domid_anonymous_bind():
    bind = partial_anon_schema([123])[0]
    given = {u'auto_domid': True}
    expected = {u'id': u'f_array'}
    assert_transform(generic.transform_domid,
                     u'input', given, expected, bind=bind)

    bind = full_anon_schema([123])[0]
    given = {u'auto_domid': True}
    expected = {}
    assert_transform(generic.transform_domid,
                     u'input', given, expected, bind=bind)


def test_domid_unbound_naming():
    given = {u'name': u'unbound',
             u'auto_domid': u'on'}
    expected = {u'id': u'f_unbound', u'name': u'unbound'}
    assert_unbound_transform(generic.transform_domid,
                             u'input', given, expected)


def test_domid_custom_formatting():
    context = Context()
    context[u'domid_format'] = u'--%s--'
    given = {u'auto_domid': u'on'}
    expected = {u'id': u'--number--'}
    assert_bound_transform(generic.transform_domid,
                           u'input', given, expected, context=context)


def test_for_anonymous_bind():
    bind = partial_anon_schema([123])[0]
    given = {u'auto_for': True}
    expected = {u'for': u'f_array'}
    assert_transform(generic.transform_for,
                     u'label', given, expected, bind=bind)

    bind = full_anon_schema([123])[0]
    given = {u'auto_for': True}
    expected = {}
    assert_transform(generic.transform_for,
                     u'label', given, expected, bind=bind)


def test_tabindex_stop_numbers():
    given = {u'auto_tabindex': u'on'}

    for stop_num in -1, -2:
        context = Context()
        context[u'tabindex'] = stop_num
        expected = {u'tabindex': unicode(stop_num)}
        assert_bound_transform(generic.transform_tabindex,
                               u'input', given, expected, context=context)
        assert_unbound_transform(generic.transform_tabindex,
                                 u'input', given, expected, context=context)
        assert context[u'tabindex'] == stop_num


def test_tabindex_zero():
    given = {}
    context = Context()
    context[u'auto_tabindex'] = True
    context[u'tabindex'] = 0

    expected = {}
    assert_unbound_transform(generic.transform_tabindex,
                             u'input', given, expected, context=context)


def test_tabindex_increment():
    given = {}
    context = Context()
    context[u'auto_tabindex'] = True
    context[u'tabindex'] = 1

    expected = {u'tabindex': u'1'}
    assert_unbound_transform(generic.transform_tabindex,
                             u'input', given, expected, context=context)

    expected = {u'tabindex': u'2'}
    assert_unbound_transform(generic.transform_tabindex,
                             u'input', given, expected, context=context)

    assert context[u'tabindex'] != 1


def test_filter():

    given = {}

    def filter1(tagname, attributes, contents, context, bind):
        attributes[u'donut'] = u'xyzzy'
        return contents

    context = Context()
    context[u'auto_filter'] = True
    context[u'filters'] = [filter1]

    expected = {
        u'donut': u'xyzzy',
        }
    assert_bound_transform(generic.transform_filters,
                           u'div', given, expected, context=context)

    def filter2(tagname, attributes, contents, context, bind):
        return bind.label

    context = Context()
    context[u'auto_filter'] = True
    context[u'filters'] = [filter2, filter1]
    contents = u'bbq'
    expected_contents = u'number'

    expected = {
        u'donut': u'xyzzy',
        }
    assert_bound_transform(generic.transform_filters,
                           u'div', given, expected, context=context,
                           contents=contents,
                           expected_contents=expected_contents)

    context = Context()
    context[u'auto_filter'] = True
    context[u'filters'] = ()
    contents = u'bbq'
    expected_contents = u'bbq'

    expected = {}
    assert_bound_transform(generic.transform_filters,
                           u'div', given, expected, context=context,
                           contents=contents,
                           expected_contents=expected_contents)


def test_filter_want():

    def filter1(tagname, attributes, contents, context, bind):
        attributes[u'donut'] = u'xyzzy'
        return contents

    filter1.tags = [u'div', u'input']

    for tag in u'div', u'input':
        context = Context()
        context[u'auto_filter'] = True
        context[u'filters'] = [filter1]
        given = {}
        expected = {
            u'donut': u'xyzzy',
            }
        assert_bound_transform(generic.transform_filters,
                           tag, given, expected, context=context)


    context = Context()
    context[u'auto_filter'] = True
    context[u'filters'] = [filter1]
    given = {}
    expected = {}
    assert_bound_transform(generic.transform_filters,
                           u'horse', given, expected, context=context)



import re

from flatland import AdaptationError, Long
from flatland.util import Unspecified
from flatland.validation import (
    Converted,
    Luhn10,
    Present,
    Validator,
)

VISA = 'Visa',
MASTERCARD = 'MasterCard',
AMEX = 'American Express',
DISCOVER = 'Discover',


class CreditCardNumber(Long):
    accepted = (VISA, MASTERCARD, AMEX)

    def __init__(self, value=Unspecified, **kw):
        validators = [
            self.Present(),
            self.Converted(),
            self.Luhn10(),
            self.AcceptedType()
        ]
        kw.setdefault('validators', validators)
        Long.__init__(self, value, **kw)

    def adapt(self, value):
        if value is None:
            return None
        elif isinstance(value, (int, long)):
            return long(value)

        value = _from_string(value)

        if value is None:
            raise AdaptationError()

        return value

    def serialize(self, value):
        if value is None:
            return u''
        elif isinstance(value, long):
            return _pretty_print(value)
        else:
            return unicode(value)

    Present = Present

    Converted = Converted

    Luhn10 = Luhn10

    class AcceptedType(Validator):
        def _formatter(element, state):
            accepted = element.accepted
            if len(accepted) > 2:
                types = (', '.join(label for label, in accepted) +
                       ', and ' + accepted[-1][0])
            elif len(accepted) == 2:
                types = ' and '.join(label for label, in accepted)
            else:
                types = accepted[0][0]

            return u'We accept %s' % types

        not_accepted = _formatter

        def validate(self, element, state):
            type_ = _card_type(element.value)
            if type_ in element.accepted:
                return True

            return self.note_error(element, state, 'not_accepted')

_re_visa = re.compile(r'^4\d{12}\d{3}?$')
_re_mc   = re.compile(r'^5[1-5]\d{14}$')
_re_amex = re.compile(r'^3[47]\d{13}$')
_re_disc = re.compile(r'^6011\d{12}$')

def _card_type(number):
    assert isinstance(number, (int, long))

    as_str = str(number)

    if _re_visa.match(as_str):
        return VISA
    elif _re_mc.match(as_str):
        return MASTERCARD
    elif _re_amex.match(as_str):
        return AMEX
    elif _re_disc.match(as_str):
        return DISCOVER
    else:
        return None

_re_strip = re.compile(r'[^0-9]')
_re_filler = re.compile(r'^[0-9\s-]{15,}$')

def _from_string(number):
    if number is None:
        return None
    elif isinstance(number, (int, long)):
        return long(number)
    elif not isinstance(number, basestring):
        return None

    if not _re_filler.match(number):
        return None

    return long(_re_strip.sub(u'', unicode(number)))

def _pretty_print(number):
    if number is None:
        return u''

    s = unicode(number)

    if len(s) == 16:
        return u'%s-%s-%s-%s' % (s[0:4], s[4:8], s[8:12], s[12:16])
    elif len(s) == 15:
        return u'%s-%s-%s' % (s[0:4], s[4:10], s[10:15])
    elif len(s) == 13:
        return u'%s-%s-%s-%s' % (s[0:4], s[4:7], s[7:10], s[10:13])
    else:
        return s



# -*- coding: utf-8; fill-column: 78 -*-
from collections import defaultdict
import re

from flatland.util import (
    Unspecified,
    assignable_class_property,
    autodocument_from_superclasses,
    class_cloner,
    keyslice_pairs,
    re_uescape,
    to_pairs,
    )
from flatland.schema.base import Element, Unevaluated, Slot, validate_element
from flatland.schema.scalars import Scalar


__all__ = (
    'Array',
    'Container',
    'Dict',
    'List',
    'Mapping',
    'MultiValue',
    'Sequence',
    )


class Container(Element):
    """Holds other schema items.

    Base class for elements that can contain other elements, such as
    :class:`List` and :class:`Dict`.

    :param descent_validators: optional, a sequence of validators that
      will be run before contained elements are validated.

    :param validators: optional, a sequence of validators that will be
      run after contained elements are validated.

    :param \*\*kw: other arguments common to
      :class:`~flatland.schema.base.FieldSchema`.

    """

    validates_down = 'descent_validators'

    validates_up = 'validators'

    descent_validators = ()
    """TODO: doc descent_validators"""

    @class_cloner
    def descent_validated_by(cls, *validators):
        """Return a class with descent validators set to *\*validators*.

        :param \*validators: one or more validator functions, replacing any
          descent validators present on the class.

        :returns: a new class
        """
        for validator in validators:
            # metaclass gymnastics can fool this assertion. don't do that.
            if isinstance(validator, type):
                raise TypeError(
                    "Validator %r is a type, not a callable or instance of a"
                    "validator class.  Did you mean %r()?" % (
                        validator, validator))
        cls.descent_validators = list(validators)
        return cls

    @class_cloner
    def including_descent_validators(cls, *validators, **kw):
        """Return a class with additional descent *\*validators*.

        :param \*validators: one or more validator functions

        :param position: defaults to -1.  By default, additional validators
          are placed after existing descent validators.  Use 0 for before, or
          any other list index to splice in *validators* at that point.

        :returns: a new class
        """
        position = kw.pop('position', -1)
        if kw:
            raise TypeError('including_validators() got an '
                            'unexpected keyword argument %r' % (
                                kw.popitem()[0]))
        mutable = list(cls.descent_validators)
        if position < 0:
            position = len(mutable) + 1 + position
        mutable[position:position] = list(validators)
        cls.descent_validators = mutable
        return cls

    def validate_element(self, element, state, descending):
        """Validates on the first (downward) and second (upward) pass.

        If :attr:`descent_validators` are defined on the schema, they
        will be evaluated before children are validated.  If a
        validation function returns :obj:`flatland.SkipAll` or
        :obj:`flatland.SkipFalse`, downward validation will halt on
        this container and children will not be validated.

        If :attr:`validators` are defined, they will be evaluated
        after children are validated.

        See :meth:`FieldSchema.validate_element`.

        """
        if descending:
            if self.descent_validators:
                return validate_element(
                    element, state, self.descent_validators)
            else:
                return None
        else:
            return validate_element(element, state, self.validators)

    def _validate(self, state, descending):
        """Run validation, transforming None into success. Internal."""
        # FIXME: refactor this to allow for this logic ("Don't apply default
        # validation on downward pass") to be defined declaratively.
        if descending:
            if self.validates_down:
                validators = getattr(self, self.validates_down, None)
                if not validators:
                    return Unevaluated
                return validate_element(self, state, validators)
        else:
            if self.validates_up:
                validators = getattr(self, self.validates_up, None)
                return validate_element(self, state, validators)
        return Unevaluated


class Sequence(Container, list):
    """Abstract base of sequence-like Containers.

    Instances of :class:`Sequence` hold other elements and operate like Python
    lists.  Each sequence member will be an instance of :attr:`member_schema`.

    Python list methods and operators may be passed instances of
    :attr:`member_schema` or plain Python values.  Using plain values is a
    shorthand for creating an :attr:`member_schema` instance and
    :meth:`set()ting<flatland.schema.base.Element.set>` it with the value:

    .. doctest::

      >>> from flatland import Array, Integer
      >>> Numbers = Array.of(Integer)
      >>> ones = Numbers()
      >>> ones.append(1)
      >>> ones
      [<Integer None; value=1>]
      >>> another_one = Integer()
      >>> another_one.set(1)
      True
      >>> ones.append(another_one)
      >>> ones
      [<Integer None; value=1>, <Integer None; value=1>]

    """

    member_schema = None
    """An :class:`~flatland.schema.base.Element` class for sequence members."""

    prune_empty = True
    """If true, skip missing index numbers in :meth:`set_flat`. Default True.

    See `Sequences`_ for more information.

    """

    def __init__(self, value=Unspecified, **kw):
        Container.__init__(self, value, **kw)
        if not self.member_schema:
            raise TypeError("Invalid schema: %r has no member_schema" %
                            type(self))

    @class_cloner
    def of(cls, *schema):
        """Declare the class to hold a sequence of *\*schema*.

        :params \*schema: one or more :class:`flatland.Element` classes
        :returns: *cls*

        Configures the :attr:`member_schema` of *cls* to hold instances of
        *\*schema*.

        .. doctest::

          >>> from flatland import Array, String
          >>> Names = Array.of(String.named('name'))
          >>> Names.member_schema
          <class 'flatland.schema.scalars.String'>
          >>> el = Names(['Bob', 'Biff'])
          >>> el
          [<String u'name'; value=u'Bob'>, <String u'name'; value=u'Biff'>]

        If more than one :class:`~flatland.Element` is specified in
        *\*schema*, an anonymous :class:`Dict` is created to hold them.

        .. doctest::

          >>> from flatland import Integer
          >>> Points = Array.of(Integer.named('x'), Integer.named('y'))
          >>> Points.member_schema
          <class 'flatland.schema.containers.Dict'>
          >>> el = Points([dict(x=1, y=2)])
          >>> el
          [{u'y': <Integer u'y'; value=2>, u'x': <Integer u'x'; value=1>}]

        """
        for field in schema:
            if isinstance(field, Element):
                raise TypeError("'of' must be initialized with types, got "
                                "instance %r" % field)
        if not schema:
            raise TypeError("One or more Element classes is required")
        elif len(schema) == 1:
            cls.member_schema = schema[0]
        else:
            cls.member_schema = Dict.of(*schema)
        return cls

    def set(self, iterable):
        """Assign the native and Unicode value.

        Attempts to adapt the given *iterable* and assigns this element's
        :attr:`value` and :attr:`u` attributes in tandem.  Returns True if the
        adaptation was successful.  See
        :meth:`Element.set()<flatland.schema.base.Element.set>`.

        Set must be supplied a Python sequence or iterable:

        .. doctest::

          >>> from flatland import Integer, List
          >>> Numbers = List.of(Integer)
          >>> nums = Numbers()
          >>> nums.set([1, 2, 3, 4])
          True
          >>> nums.value
          [1, 2, 3, 4]

        """

        del self[:]
        self.raw = iterable
        values, converted = [], True
        try:
            for v in iterable:
                el = self.member_schema()
                converted &= el.set(v)
                values.append(el)
            self.extend(values)
        except TypeError:
            return False
        else:
            return converted

    def set_default(self):
        default = self.default_value
        if default is not None and default is not Unspecified:
            del self[:]
            self.extend(default)

    def _set_flat(self, pairs, sep):
        raise NotImplementedError()

    @property
    def children(self):
        return iter(self)

    @property
    def is_empty(self):
        return not any(True for _ in self.children)

    def _index(self, name):
        try:
            idx = int(name)
        except ValueError:
            raise IndexError(name)
        return self[idx]

    def append(self, value):
        """Append *value* to end.

        If *value* is not an instance of :attr:`member_schema`, it will be
        wrapped in a new element of that type before appending.

        """
        if not isinstance(value, Element):
            value = self.member_schema(value=value)
        value.parent = self
        list.append(self, value)

    def extend(self, iterable):
        """Append *iterable* values to the end.

        If values of *iterable* are not instances of :attr:`member_schema`,
        they will be wrapped in a new element of that type before extending.

        """
        for value in iterable:
            self.append(value)

    def insert(self, index, value):
        """Insert *value* at *index*.

        If *value* is not an instance of :attr:`member_schema`, it will be
        wrapped in a new element of that type before inserting.

        """
        if not isinstance(value, Element):
            value = self.member_schema(value=value)
        value.parent = self
        list.insert(self, index, value)

    def __setitem__(self, index, value):
        if isinstance(index, slice):
            as_elements = []
            for item in value:
                if not isinstance(item, Element):
                    item = self.member_schema(value=item)
                item.parent = self
                as_elements.append(item)
            value = as_elements
        else:
            if not isinstance(value, Element):
                value = self.member_schema(value=value)
                value.parent = self
        list.__setitem__(self, index, value)

    def __setslice__(self, i, j, value):
        self.__setitem__(slice(i, j), value)

    def remove(self, value):
        """Remove member with value *value*.

        If *value* is not an instance of :attr:`member_schema`, it will be
        wrapped in a new element of that type before searching for a matching
        element to remove.

        """
        if not isinstance(value, Element):
            value = self.member_schema(value=value)
        list.remove(self, value)

    def index(self, value):
        """Return first index of *value*.

        If *value* is not an instance of :attr:`member_schema`, it will be
        wrapped in a new element of that type before searching for a matching
        element in the sequence.

        """
        if not isinstance(value, Element):
            value = self.member_schema(value=value)
        return list.index(self, value)

    def count(self, value):
        """Return number of occurrences of *value*.

        If *value* is not an instance of :attr:`member_schema`, it will be
        wrapped in a new element of that type before searching for matching
        elements in the sequence.

        """
        if not isinstance(value, Element):
            value = self.member_schema(value=value)
        return list.count(self, value)

    def __contains__(self, value):
        """Return True if sequence contains *value*.

        If *value* is not an instance of :attr:`member_schema`, it will be
        wrapped in a new element of that type before searching for a matching
        element in the sequence.

        """
        if not isinstance(value, Element):
            value = self.member_schema(value=value)
        return list.__contains__(self, value)

    @property
    def value(self):
        return list(value.value for value in self.children)

    @property
    def u(self):
        return u'[%s]' % u', '.join(
            element.u if isinstance(element, Container)
                      else repr(element.u).decode('raw_unicode_escape')
            for element in self.children)


class ListSlot(Container, Slot):
    """Wraps elements of Lists & models their position in the list.

    :class:`List ` makes these mostly invisible to the outside, appearing only
    when flattening names.  The :attr:`name` is set by the List and will be a
    unicoded integer index.  Flattening a list name will join the parent's
    name with the slot's name with the child element's name:

      'listname_0_childname', 'listname_1_childname'

    """

    def __init__(self, name, parent, element):
        self.name = name
        self.parent = parent
        self.element = element
        element.parent = self

    @property
    def u(self):
        return self.element.u

    @property
    def value(self):
        return self.element.value

    def __repr__(self):
        return '<ListSlot[%r] for %r>' % (self.name, self.element)


class List(Sequence):
    """An ordered, homogeneous Sequence."""

    # TODO: clarify if descent_validators run on empty, optional sequences

    slot_type = ListSlot

    # Default definition duplicated for sphinx documentation purposes
    member_schema = ()
    """An :class:`~flatland.schema.base.Element` class for member elements.

    See also the :meth:`~Sequence.of` schema configuration method.

    """

    maximum_set_flat_members = 1024
    """Maximum list members set in a :meth:`set_flat` operation.

    Once this maximum of child members has been added, subsequent data will be
    dropped.  This ceiling prevents denial of service attacks when processing
    Lists with :attr:`prune_empty` set to False; without it remote attackers
    can trivially exhaust memory by specifying one low and one very high
    index.

    """

    def _as_element(self, value):
        """TODO"""
        if value is Unspecified:
            return self.member_schema()
        if isinstance(value, Element):
            return value
        else:
            return self.member_schema(value)

    def _new_slot(self, value=Unspecified):
        """Wrap *value* in a Slot named as the element's index in the list."""
        return self.slot_type(name=str(len(self)).decode('ascii'),
                              parent=self,
                              element=self._as_element(value))

    @property
    def _slots(self):
        """An iterator of the List's otherwise hidden Slots."""
        return list.__iter__(self)

    def append(self, value):
        list.append(self, self._new_slot(value))

    def extend(self, iterable):
        for v in iterable:
            self.append(v)

    def __getitem__(self, index):
        if isinstance(index, slice):
            return [item.element for item in list.__getitem__(self, index)]
        return list.__getitem__(self, index).element

    def __getslice__(self, i, j):
        return self.__getitem__(slice(i, j))

    def __setitem__(self, index, value):
        if isinstance(index, slice):
            value = [self._new_slot(item) for item in value]
            list.__setitem__(self, index, value)
            self._renumber()
        else:
            slot = self[index]
            slot.set(value)

    def __setslice__(self, i, j, sequence):
        return self.__setitem__(slice(i, j), sequence)

    def __iter__(self):
        for i in list.__iter__(self):
            yield i.element

    def __delitem__(self, index):
        # Optimizing __delitem__ or pop when removing only the last item
        # doesn't seem worth it.
        list.__delitem__(self, index)  # slices ok
        self._renumber()

    def __delslice__(self, i, j):
        return self.__delitem__(slice(i, j))

    def pop(self, index=-1):
        value = list.pop(self, index)
        self._renumber()
        value.parent = None
        return value

    def insert(self, index, value):
        list.insert(self, index, self._new_slot(value))
        self._renumber()

    def remove(self, value):
        list.remove(self, self._as_element(value))
        self._renumber()

    def sort(self, cmp=None, key=None, reverse=False):
        list.sort(self, cmp, key, reverse)
        self._renumber()

    def reverse(self):
        list.reverse(self)
        self._renumber()

    def _renumber(self):
        for idx, slot in enumerate(self._slots):
            slot.name = str(idx).decode('ascii')

    @property
    def children(self):
        return iter(child.element for child in self._slots)

    def _set_flat(self, pairs, sep):
        del self[:]

        if not pairs:
            return

        if self.name:
            regex = re.compile(ur'^%s(\d+)(?:%s|$)' % (
                re_uescape(self.name + sep), re_uescape(sep)), re.UNICODE)
        else:
            regex = re.compile(ur'^(\d+)(?:%s|$)' % (
                re_uescape(sep)), re.UNICODE)

        indexes = defaultdict(list)
        prune = self.prune_empty

        for key, value in pairs:
            if value == u'' and prune:
                continue
            m = regex.match(key)
            if not m:
                continue
            try:
                index = long(m.group(1))
            except TypeError:
                # Ignore keys with outrageously large indexes- they
                # aren't valid data for us.
                pass
            else:
                child_key = key[len(m.group(0)):] or None
                indexes[index].append((child_key, value))

        if not indexes:
            return

        # lossy: missing (or empty-valued) indexes are omitted.
        #        the python indexes may not match the flat indexes
        if prune:
            for offset, index in enumerate(sorted(indexes)):
                if offset == self.maximum_set_flat_members:
                    break
                slot = self._new_slot()
                list.append(self, slot)
                slot.element.set_flat(indexes[index], sep)
        # lossless: elements are built up to the highest seen index or a
        #           schema-configured maximum. flat + python indexes match.
        else:
            max_index = min(max(indexes) + 1, self.maximum_set_flat_members)
            for index in xrange(0, max_index):
                slot = self._new_slot()
                list.append(self, slot)
                flat = indexes.get(index, None)
                if flat:
                    slot.element.set_flat(flat, sep)

    def set_default(self):
        """set() the element to the schema default.

        List's set_default supports two modes for
        :attr:`~flatland.schema.base.Element.default` values:

        - If default is an integer, the List will be filled with that many
          elements.  Each element will then have
          :meth:`~flatland.schema.base.Element.set_default` called on it.

        - Otherwise if default has a value, the list will be :meth:`set` with
          it.

        """
        default = self.default_value
        if default is None or default is Unspecified:
            return

        del self[:]
        if isinstance(default, int):
            for _ in xrange(0, default):
                slot = self._new_slot()
                list.append(self, slot)
                slot.element.set_default()
        else:
            self.set(default)

    def __repr__(self):
        # shield the slots from repr
        return repr(list(self))


class Array(Sequence):
    """A transparent homogeneous Container, for multivalued form elements.

    Arrays hold a collection of values under a single name, allowing
    all values of a repeated `(key, value)` pair to be captured and
    used.  Elements are sequence-like.

    """

    prune_empty = True
    flattenable = False

    def _set_flat(self, pairs, sep):
        del self[:]
        prune = self.prune_empty
        child_name = self.member_schema.name

        # TODO: some complexity snuck in below with the thought of supporting
        # arrays of containers.  they're *not* working yet.
        assert not issubclass(self.member_schema, Container), \
               "Flattened Arrays are only supported for scalar child types."

        if not self.name:
            child_prefix = child_name or u''
            for key, value in pairs:
                if prune and value == u'' and key == child_prefix:
                    continue
                if key == u'':
                    key = None
                if child_name and key != child_name:
                    continue
                member = self.member_schema.from_flat([(key, value)])
                self.append(member)
        else:
            regex = re.compile(ur'^(%s(?:%s|$))' % (
                re_uescape(self.name), re_uescape(sep)), re.UNICODE)
            for key, value in pairs:
                m = regex.match(key)
                if not m:
                    continue
                remainder = key[m.end():] or None
                if child_name and not remainder:
                    continue
                elif prune and value == u'' and remainder == child_name:
                    continue
                member = self.member_schema.from_flat([(remainder, value)])
                self.append(member)


class MultiValue(Array, Scalar):
    """A transparent homogeneous Container, for multivalued form elements.

    MultiValues combine aspects of :class:`Scalar` and
    :class:`Sequence` fields, allowing all values of a repeated `(key,
    value)` pair to be captured and used.

    MultiValues take on the name of their child and have no identity
    of their own when flattened.  Elements are mostly sequence-like
    and can be indexed and iterated. However the :attr:`.u` or
    :attr:`.value` are scalar-like, and return values from the first
    element in the sequence.

    """

    def u(self):
        """The .u of the first item in the sequence, or u''."""
        if not self:
            return u''
        else:
            return self[0].u

    def _set_u(self, value):
        if not self:
            self.append(None)
        self[0].u = value

    u = property(u, _set_u)
    del _set_u

    def value(self):
        """The .value of the first item in the sequence, or None."""
        if not self:
            return None
        else:
            return self[0].value

    def _set_value(self, value):
        if not self:
            self.append(None)
        self[0].value = value

    value = property(value, _set_value)
    del _set_value

    def __nonzero__(self):
        # this is a little troubling, given that it may not match the
        # appearance of the element in a scalar context.
        return len(self)


class Mapping(Container, dict):
    """Base of mapping-like Containers."""

    field_schema = ()
    """TODO: doc field_schema"""

    def __init__(self, value=Unspecified, **kw):
        Container.__init__(self, **kw)
        if not self.field_schema:
            raise TypeError("%r dictionary type has no fields defined" % (
                type(self).__name__))
        self._reset()
        if value is not Unspecified:
            self.set(value)

    def __setitem__(self, key, value):
        if not key in self:
            raise TypeError('May not set unknown key %r on %s %r' %
                            (key, type(self).__name__, self.name))
        self[key].set(value)

    def __delitem__(self, key):
        # this may be overly pedantic
        if key not in self:
            raise KeyError(key)
        raise TypeError('%s keys are immutable.' % type(self).__name__)

    def may_contain(self, key):
        """Return True if the element schema allows a field named **key**."""
        return key in self

    def clear(self):
        raise TypeError('%s keys are immutable.' % type(self).__name__)

    def _reset(self):
        """Place blank children in all fields."""
        for member_schema in self.field_schema:
            key = member_schema.name
            dict.__setitem__(
                self, key, member_schema(parent=self))

    def popitem(self):
        raise TypeError('%s keys are immutable.' % type(self).__name__)

    def pop(self, key):
        if key not in self:
            raise KeyError(key)
        raise TypeError('%s keys are immutable.' % type(self).__name__)

    def update(self, *dictish, **kwargs):
        """Update with keys from dict-like *\*dictish* and *\*\*kwargs*"""
        if len(dictish) > 1:
            raise TypeError(
                "update expected at most 1 arguments, got %s" % len(dictish))
        elif dictish:
            for key, value in to_pairs(dictish[0]):
                self[key] = value
        for key, value in kwargs.iteritems():
            self[key] = value

    def setdefault(self, key, default=None):
        # The key will always either be present or not creatable.
        raise TypeError('%s keys are immutable.' % type(self).__name__)

    def get(self, key, default=None):
        if key not in self:
            raise KeyError(
                'immutable %s %s schema does not contain key %r.' % (
                    type(self).__name__, self.name, key))
        # default will never be used.
        return self[key]

    @property
    def children(self):
        # order not guaranteed
        return self.itervalues()

    def set(self, value):
        """TODO: doc set()"""
        self.raw = value
        pairs = to_pairs(value)
        self._reset()

        seen = set()
        converted = True
        for key, value in pairs:
            if not self.may_contain(key):
                raise KeyError(
                    '%s %r schema does not allow key %r' % (
                        type(self).__name__, self.name, key))
            converted &= self[key].set(value)
            seen.add(key)
        required = set(self.iterkeys())
        if seen != required:
            missing = required - seen
            raise TypeError(
                'all keys required for a set() operation, missing %s.' % (
                    ','.join(repr(key) for key in missing)))
        return converted

    def _set_flat(self, pairs, sep):
        if self.name is None:
            possibles = pairs  # accept all
        else:
            possibles = []
            prefix = self.name + sep
            plen = len(prefix)
            for key, value in pairs:
                if key == prefix:
                    # No flat representation of mappings, ignore.
                    pass
                if key.startswith(prefix):
                    # accept child element
                    possibles.append((key[plen:], value))

        if not possibles:
            return

        for schema in self.field_schema:
            field = schema.name
            accum = []
            for key, value in possibles:
                if key.startswith(field):
                    accum.append((key, value))
            if accum:
                if dict.__contains__(self, field):
                    self[field].set_flat(accum, sep)
                else:
                    self[field] = schema()
                    self[field].set_flat(accum, sep=sep)

    def set_default(self):
        default = self.default_value
        if default is not None and default is not Unspecified:
            self.set(default)
        else:
            for child in self.children:
                child.set_default()

    def _index(self, name):
        return self[name]

    @property
    def u(self):
        """A string repr of the element."""
        pairs = ((key, value.u if isinstance(value, Container)
                               else repr(value.u).decode('raw_unicode_escape'))
                  for key, value in self.iteritems())
        return u'{%s}' % u', '.join(
            u"%s: %s" % (repr(k).decode('raw_unicode_escape'), v)
            for k, v in pairs)

    @property
    def value(self):
        """The element as a regular Python dictionary."""
        return dict((key, value.value) for key, value in self.iteritems())

    @property
    def is_empty(self):
        """Mappings are never empty."""
        return False

    @assignable_class_property
    def field_schema_mapping(instance, cls):
        """A name -> schema mapping generated from :attr:`field_schema`."""
        if instance is not None:
            field_schema = instance.field_schema
        else:
            field_schema = cls.field_schema
        return dict((schema.name, schema) for schema in field_schema)

    def _field_schema_for(self, key):
        """Return the schema for field ``*key* or None."""
        for schema in self.field_schema:
            if schema.name == key:
                return schema
        return None


class Dict(Mapping, dict):
    """A mapping Container with named members."""

    policy = 'subset'
    """One of 'strict', 'subset' or 'duck'.  Default 'subset'.

    See :ref:`set_policy`
    """

    @class_cloner
    def of(cls, *fields):
        """TODO: doc of()"""
        # TODO: doc
        # TODO: maybe accept **kw?
        for field in fields:
            if isinstance(field, Element):
                raise TypeError("'of' must be initialized with types, got "
                                "instance %r" % field)

        unique_names = set(field.name for field in fields)
        # TODO: ensure these are types, not instances
        if len(unique_names) != len(fields):
            names = [field.name for field in fields]
            dupes = [name for name in unique_names if names.count(name) > 1]
            raise TypeError(
                "All fields in a %s must have unique names. "
                "Duplicates of: %s " % (
                    cls.__name__,
                    ', '.join(repr(f) for f in dupes)))

        cls.field_schema = fields
        return cls

    @classmethod
    def from_object(cls, obj, include=None, omit=None, rename=None, **kw):
        """Return an element initialized with an object's attributes.

        :param obj: any object
        :param include: optional, an iterable of attribute names to pull from
            *obj*, if present on the object.  Only these attributes will be
            included.
        :param omit: optional, an iterable of attribute names to ignore on
            **obj**.  All other attributes matching a named field on the Form
            will be included.
        :param rename: optional, a mapping of attribute-to-field name
            transformations.  Attributes specified in the mapping will be
            included regardless of *include* or *omit*.
        :param \*\*kw: keyword arguments will be passed to the element's
            constructor.

        *include* and *omit* are mutually exclusive.

        This is a convenience constructor for :meth:`set_by_object`::

          element = cls(**kw)
          element.set_by_object(obj, include, omit, rename)

        """
        self = cls(**kw)
        self.set_by_object(obj=obj, include=include, omit=omit, rename=rename)
        return self

    def set(self, value, policy=None):
        """TODO: doc set()"""
        self.raw = value
        pairs = to_pairs(value)
        self._reset()

        if policy is not None:
            assert policy in ('strict', 'subset', 'duck')
        else:
            policy = self.policy

        fields = self.field_schema_mapping
        seen = set()
        converted = True
        for key, value in pairs:
            if key not in fields:
                if policy != 'duck':
                    raise KeyError(
                        'Dict %r schema does not allow key %r' % (
                            self.name, key))
                continue
            if dict.__contains__(self, key):
                converted &= self[key].set(value)
            else:
                self[key] = el = fields[key]()
                converted &= el.set(value)
            seen.add(key)

        if policy == 'strict':
            required = set(fields.iterkeys())
            if seen != required:
                missing = required - seen
                raise TypeError(
                    'strict-mode Dict requires all keys for '
                    'a set() operation, missing %s.' % (
                        ','.join(repr(key) for key in missing)))
        return converted

    def set_by_object(self, obj, include=None, omit=None, rename=None):
        """Set fields with an object's attributes.

        :param obj: any object
        :param include: optional, an iterable of attribute names to pull from
            *obj*, if present on the object.  Only these attributes will be
            included.
        :param omit: optional, an iterable of attribute names to ignore on
            **obj**.  All other attributes matching a named field on the Form
            will be included.
        :param rename: optional, a mapping of attribute-to-field name
            transformations.  Attributes specified in the mapping will be
            included regardless of *include* or *omit*.

        *include* and *omit* are mutually exclusive.

        Sets fields on *self*, using as many attributes as possible from
        *obj*.  Object attributes that do not correspond to field names are
        ignored.

        Mapping instances have two corresponding methods useful for
        round-tripping values in and out of your domain objects.

        .. testsetup::

          # FIXME
          from flatland import Form, String
          class UserForm(Form):
              login = String
              password = String
              verify_password = String

          class User(object):
              def __init__(self, login=None, password=None):
                  self.login = login
                  self.password = password

        :meth:`update_object` performs the inverse of :meth:`set_object`, and
        :meth:`slice` is useful for constructing new objects.

        .. doctest::

          >>> user = User('biff', 'secret')
          >>> form = UserForm()
          >>> form.set_by_object(user)
          >>> form['login'].value
          u'biff'
          >>> form['password'] = u'new-password'
          >>> form.update_object(user, omit=['verify_password'])
          >>> user.password
          u'new-password'
          >>> user_keywords = form.slice(omit=['verify_password'], key=str)
          >>> sorted(user_keywords.keys())
          ['login', 'password']
          >>> new_user = User(**user_keywords)

        """
        fields = set(self.iterkeys())
        attributes = fields.copy()
        if rename:
            rename = list(to_pairs(rename))
            attributes.update(key for key, value in rename
                                  if value in attributes)
        if omit:
            omit = list(omit)
            attributes.difference_update(omit)

        possible = ((attr, getattr(obj, attr))
                    for attr in attributes
                    if hasattr(obj, attr))

        sliced = keyslice_pairs(possible, include=include,
                                omit=omit, rename=rename)
        final = dict((key, value)
                     for key, value in sliced
                     if key in fields)
        self.set(final)

    def update_object(self, obj, include=None, omit=None, rename=None,
                      key=str):
        """Update an object's attributes using the element's values.

        Produces a :meth:`slice` using *include*, *omit*, *rename* and
        *key*, and sets the selected attributes on *obj* using
        ``setattr``.

        :returns: nothing. *obj* is modified directly.

        """
        data = self.slice(include=include, omit=omit, rename=rename, key=key)
        for attribute, value in data.iteritems():
            setattr(obj, attribute, value)

    def slice(self, include=None, omit=None, rename=None, key=None):
        """Return a ``dict`` containing a subset of the element's values."""
        return dict(
            keyslice_pairs(
                ((key, element.value) for key, element in self.iteritems()),
                include=include, omit=omit, rename=rename, key=key))


class SparseDict(Dict):
    """A Mapping which may contain a subset of the schema's allowed keys.

    This differs from :class:`Dict` in that new instances are not created with
    empty values for all possible keys.  In addition, mutating operations are
    allowed so long as the operations operate within the schema.  For example,
    you may :meth:`pop` and ``del`` members of the mapping.

    """

    #: The subset of fields to autovivify on instantiation.
    #:

    #: May be ``None`` or ``'required'``.  If ``None``, mappings will be
    #: created empty and mutation operations are unrestricted within the
    #: bounds of the :attr:`field_schema`.  If ``required``, fields with
    #: :attr:`optional` of ``False`` will always be present after
    #: instantiation, and attempts to remove them from the mapping with ``del``
    #: and friends will raise ``TypeError``.
    minimum_fields = None  # 'required'

    def may_contain(self, key):
        return key in self or self._field_schema_for(key) is not None

    def _reset(self):
        dict.clear(self)
        for member_schema in self.field_schema:
            key = member_schema.name
            if self.minimum_fields is None or member_schema.optional:
                continue
            dict.__setitem__(
                self, key, member_schema(parent=self))

    def __setitem__(self, key, value):
        schema = self._field_schema_for(key)
        if not dict.__contains__(self, key):
            if schema is None:
                raise TypeError('May not set unknown key %r on %s %r' %
                                (key, type(self).__name__, self.name))
            elif isinstance(value, schema):
                value.parent = self
                dict.__setitem__(self, key, value)
                return
            dict.__setitem__(self, key, schema(value, parent=self))
        elif isinstance(value, schema):
            value.parent = self
            dict.__setitem__(self, key, value)
        else:
            self[key].set(value)

    def __delitem__(self, key):
        if self.minimum_fields is None:
            try:
                dict.__delitem__(self, key)
                return
            except KeyError:
                if not self.may_contain(key):
                    raise TypeError(
                        'May not request del for unknown key %r on %s %r' %
                        (key, type(self).__name__, self.name))
                raise
        if key in self:
            optional = self[key].optional
        else:
            schema = self._field_schema_for(key)
            if schema is None:
                raise TypeError(
                    'May not request del for unknown key %r on %s %r' %
                    (key, type(self).__name__, self.name))
            optional = schema.optional
        if not optional:
            raise TypeError('May not delete required key %r on %s %r' %
                            (key, type(self).__name__, self.name))
        dict.__delitem__(self, key)

    def clear(self):
        self._reset()

    def popitem(self):
        raise NotImplementedError

    def pop(self, key):
        if key not in self:
            raise KeyError(key)
        if self.minimum_fields == 'required' and not self[key].optional:
            raise TypeError('May not pop required key %r on %s %r' %
                            (key, type(self).__name__, self.name))
        return dict.pop(self, key)

    def setdefault(self, key, default=None):
        if not self.may_contain(key):
            raise TypeError('Key %r not allowed in %s %r' %
                            (key, type(self).__name__, self.name))

        if key in self:
            child = self[key]
            if not child.is_empty:
                return child.value
        else:
            self[key] = child = self.field_schema_mapping[key]()
        child.set(default)
        return default

    @property
    def is_empty(self):
        for _ in self.iterkeys():
            return False
        return True

    def set_default(self):
        default = self.default_value
        if default is not None and default is not Unspecified:
            self.set(default)
        elif self.minimum_fields is None:
            self._reset()
        elif self.minimum_fields == 'required':
            self._reset()
            for schema in self.field_schema:
                if schema.optional:
                    continue
                self[schema.name] = schema.from_defaults()
        else:
            raise RuntimeError("Unknown minimum_fields setting %r" %
                               (self.minimum_fields,))


for cls_name in __all__:
    autodocument_from_superclasses(globals()[cls_name])
del cls_name



from flatland import (
    Array,
    Dict,
    Integer,
    List,
    String,
    )
from flatland.validation import (
    HasAtLeast,
    HasAtMost,
    HasBetween,
    NotDuplicated,
    )

from tests._util import assert_raises


def valid_of_children(element):
    return [e.valid for e in element.children]


def validated_string(*validators):
    return List.named('test').of(String.using(name=u'foo',
                                             validators=validators))

def test_no_duplicates_message():
    schema = validated_string(NotDuplicated())
    el = schema([u'foo', u'foo'])
    assert not el.validate()
    assert el[1].errors == [u'foo may not be repeated within test.']


def test_no_duplicates_context():
    el = String(validators=[NotDuplicated()])
    assert_raises(TypeError, el.validate)


def test_no_duplicates_comparator():
    comparator = lambda a, b: True
    schema = validated_string(NotDuplicated(comparator=comparator))
    el = schema([u'a', u'b'])
    assert not el.validate()


def _test_no_duplicates(schema, a, b):
    label = schema.label
    el = schema([a, b])
    assert el.validate()
    assert valid_of_children(el) == [True, True]

    el = schema([a, b, a])
    assert not el.validate()
    assert valid_of_children(el) == [True, True, False]
    assert el[2].errors == [u'%s %s' % (label, 3)]

    el = schema([a, b, a, b])
    assert not el.validate()
    assert valid_of_children(el) == [True, True, False, False]
    assert el[2].errors == [u'%s %s' % (label, 3)]
    assert el[3].errors == [u'%s %s' % (label, 4)]

    el = schema([a, a, a, a])
    assert not el.validate()
    assert valid_of_children(el) == [True, False, False, False]
    assert el[1].errors == [u'%s %s' % (label, 2)]
    assert el[2].errors == [u'%s %s' % (label, 3)]
    assert el[3].errors == [u'%s %s' % (label, 4)]


def test_no_duplicates_list_scalar():
    nd = NotDuplicated(failure=u'%(container_label)s %(position)s')
    schema = validated_string(nd)
    _test_no_duplicates(schema, u'foo', u'bar')


def test_no_duplicates_array():
    nd = NotDuplicated(failure=u'%(container_label)s %(position)s')
    schema = validated_string(nd)
    _test_no_duplicates(schema, u'foo', u'bar')


def test_no_duplicates_list_anon_dict():
    nd = NotDuplicated(failure=u'%(container_label)s %(position)s')
    schema = (List.named('test').
              of(Dict.of(Integer.named('x'),
                         Integer.named('y')).
                 using(validators=[nd])))
    _test_no_duplicates(schema, {'x': 1, 'y': 2}, {'x': 3, 'y': 4})


def test_no_duplicates_list_dict():
    nd = NotDuplicated(failure=u'%(container_label)s %(position)s')
    schema = (List.named('test').
              of(Dict.named('xyz').
                 of(Integer.named('x'),
                    Integer.named('y')).
                 using(validators=[nd])))
    _test_no_duplicates(schema, {'x': 1, 'y': 2}, {'x': 3, 'y': 4})


def validated_list(*validators):
    return List.named('outer').of(String.named('inner')).using(
        validators=validators)


def test_has_at_least_none():
    schema = validated_list(HasAtLeast(minimum=0))

    el = schema(['a'])
    assert el.validate()

    el = schema()
    assert el.validate()


def test_has_at_least_one():
    schema = validated_list(HasAtLeast())

    el = schema()
    assert not el.validate()
    assert el.errors == [u'outer must contain at least one inner']

    el = schema(['a'])
    assert el.validate()


def test_has_at_least_two():
    schema = validated_list(HasAtLeast(minimum=2))

    el = schema(['a'])
    assert not el.validate()
    assert el.errors == [u'outer must contain at least 2 inners']

    el = schema(['a', 'b'])
    assert el.validate()


def test_has_at_most_none():
    schema = validated_list(HasAtMost(maximum=0))

    el = schema(['a'])
    assert not el.validate()
    assert el.errors == [u'outer must contain at most 0 inners']

    el = schema()
    assert el.validate()


def test_has_at_most_one():
    schema = validated_list(HasAtMost(maximum=1))

    el = schema()
    assert el.validate()

    el = schema(['a'])
    assert el.validate()

    el = schema(['a', 'b'])
    assert not el.validate()
    assert el.errors == [u'outer must contain at most one inner']


def test_has_at_most_two():
    schema = validated_list(HasAtMost(maximum=2))

    el = schema()
    assert el.validate()

    el = schema(['a', 'b'])
    assert el.validate()

    el = schema(['a', 'b', 'c'])
    assert not el.validate()
    assert el.errors == [u'outer must contain at most 2 inners']


def test_has_between_none():
    schema = validated_list(HasBetween(minimum=0, maximum=0))

    el = schema()
    assert el.validate()

    el = schema(['a'])
    assert not el.validate()
    assert el.errors == [u'outer must contain exactly 0 inners']


def test_has_between_one():
    schema = validated_list(HasBetween(minimum=1, maximum=1))

    el = schema()
    assert not el.validate()
    assert el.errors == [u'outer must contain exactly one inner']

    el = schema(['a'])
    assert el.validate()

    el = schema(['a', 'b'])
    assert not el.validate()
    assert el.errors == [u'outer must contain exactly one inner']


def test_has_between_two():
    schema = validated_list(HasBetween(minimum=2, maximum=2))

    el = schema()
    assert not el.validate()
    assert el.errors == [u'outer must contain exactly 2 inners']

    el = schema(['b'])
    assert not el.validate()
    assert el.errors == [u'outer must contain exactly 2 inners']

    el = schema(['a', 'b'])
    assert el.validate()


def test_has_between_none_and_one():
    schema = validated_list(HasBetween(minimum=0, maximum=1))

    el = schema()
    assert el.validate()

    el = schema(['b'])
    assert el.validate()

    el = schema(['a', 'b'])
    assert not el.validate()
    assert el.errors == [u'outer must contain at least 0 and at most 1 inner']


def test_has_between_none_and_two():
    schema = validated_list(HasBetween(minimum=0, maximum=2))

    el = schema()
    assert el.validate()

    el = schema(['a', 'b'])
    assert el.validate()

    el = schema(['a', 'b', 'c'])
    assert not el.validate()
    assert el.errors == [u'outer must contain at least 0 and at most 2 inners']


def test_has_between_one_and_two():
    schema = validated_list(HasBetween(minimum=1, maximum=2))

    el = schema()
    assert not el.validate()
    assert el.errors == [u'outer must contain at least 1 and at most 2 inners']

    el = schema(['b'])
    assert el.validate()

    el = schema(['a', 'b'])
    assert el.validate()

    el = schema(['a', 'b', 'c'])
    assert not el.validate()
    assert el.errors == [u'outer must contain at least 1 and at most 2 inners']



from datetime import date, timedelta

from flatland import (
    Date,
    Dict,
    Form,
    Integer,
    List,
    String,
    )
from flatland.validation import (
    IsFalse,
    IsTrue,
    MapEqual,
    UnisEqual,
    ValueAtLeast,
    ValueAtMost,
    ValueBetween,
    ValueGreaterThan,
    ValueIn,
    ValueLessThan,
    ValuesEqual,
    )

from tests._util import eq_


def form(value):

    class Schema(Form):
        name = 'test'
        x = String
        y = String
        z = String
        sub = Dict.of(String.named('xx'))

    return Schema(value)


def scalar(value):
    return String(value, name='test')


def integer_scalar(value):
    return Integer(value, name='test')


def date_scalar(value):
    return Date(value, name='test')


def test_is_true():
    i = integer_scalar(1)
    z = integer_scalar(0)
    v = IsTrue()
    assert v.validate(i, None)
    assert not v.validate(z, None)


def test_is_false():
    i = integer_scalar(1)
    z = integer_scalar(0)
    v = IsFalse()
    assert not v.validate(i, None)
    assert v.validate(z, None)


def test_value_in():
    v = ValueIn(('a', 'b', 'c'))

    for good_val in ('a', 'b', 'c'):
        s = scalar(good_val)
        assert v.validate(s, None)

    for bad_val in (None, 'x', -1):
        s = scalar(bad_val)
        assert not v.validate(s, None)


def test_value_less_than():
    i = integer_scalar(1)
    V = ValueLessThan
    assert V(2).validate(i, None)
    assert not V(1).validate(i, None)
    assert i.errors == ['test must be less than 1.']

    d = date_scalar(date.today())
    assert V(date.today() + timedelta(days=2)).validate(d, None)
    two_days_ago = date.today() - timedelta(days=2)
    assert not V(two_days_ago).validate(d, None)
    assert d.errors == ['test must be less than %s.' % two_days_ago]


def test_value_at_most():
    i = integer_scalar(1)
    V = ValueAtMost
    assert V(2).validate(i, None)
    assert V(1).validate(i, None)
    assert not V(0).validate(i, None)
    assert i.errors == ['test must be less than or equal to 0.']


def test_value_greater_than():
    i = integer_scalar(1)
    V = ValueGreaterThan
    assert V(0).validate(i, None)
    assert not V(1).validate(i, None)
    assert i.errors == ['test must be greater than 1.']


def test_value_at_least():
    i = integer_scalar(1)
    V = ValueAtLeast
    assert V(0).validate(i, None)
    assert V(1).validate(i, None)
    assert not V(2).validate(i, None)
    assert i.errors == ['test must be greater than or equal to 2.']


def test_value_between():
    i = integer_scalar(1)
    V = ValueBetween
    assert V(0, 2).validate(i, None)
    assert V(0, 1, inclusive=True).validate(i, None)
    assert V(1, 2, inclusive=True).validate(i, None)
    assert not V(2, 3).validate(i, None)
    assert i.errors == ['test must be in the range 2 to 3.']
    i.errors = []
    assert not V(-1, 0).validate(i, None)
    assert i.errors == ['test must be in the range -1 to 0.']
    i.errors = []
    assert not V(1, 2, inclusive=False).validate(i, None)
    assert i.errors == ['test must be greater than 1 and less than 2.']
    i.errors = []
    assert not V(0, 1, inclusive=False).validate(i, None)
    assert i.errors == ['test must be greater than 0 and less than 1.']


def test_map_equal():
    v = MapEqual('x', 'y',
                         transform=lambda el: el.value.upper(),
                         unequal='%(labels)s/%(last_label)s')
    el = form(dict(x='a', y='A'))
    assert v.validate(el, None)

    el = form(dict(x='a', y='B'))
    assert not v.validate(el, None)
    eq_(el.errors, ['x/y'])


def test_values_equal_two():
    v = ValuesEqual('x', 'y')
    el = form(dict(x='a', y='a', z='a'))
    assert v.validate(el, None)

    el = form(dict(x='a', y='b', z='c'))
    assert not v.validate(el, None)
    eq_(el.errors, ['x and y do not match.'])

    el = form(dict(x='a'))
    assert not v.validate(el, None)
    eq_(el.errors, ['x and y do not match.'])


def test_values_equal_three():
    v = ValuesEqual('x', 'y', 'z')
    el = form(dict(x='a', y='a', z='a'))
    assert v.validate(el, None)

    el = form(dict(x='a', y='b', z='c'))
    assert not v.validate(el, None)
    eq_(el.errors, ['x, y and z do not match.'])

    el = form(dict(x='a'))
    assert not v.validate(el, None)
    eq_(el.errors, ['x, y and z do not match.'])


def test_values_equal_resolution():
    v = ValuesEqual('x', '/sub/xx')
    el = form(dict(x='a', sub=dict(xx='a')))
    assert v.validate(el, None)

    v = ValuesEqual('/x', 'xx')
    el = form(dict(x='a', sub=dict(xx='a')))
    assert v.validate(el['sub'], None)

    # unhashable
    v = ValuesEqual('a', 'b')
    schema = Dict.of(List.named('a').of(String.named('x')),
                     List.named('b').of(String.named('x')))

    el = schema(dict(a=['a', 'b'], b=['a', 'b']))
    assert v.validate(el, None)

    el = schema(dict(a=['a', 'b'], b=['x', 'y']))
    assert not v.validate(el, None)


def test_unis_equal():
    schema = Dict.of(String.named('s'), Integer.named('i'))
    el = schema(dict(s='abc', i='abc'))

    v = UnisEqual('s', 'i')
    assert v.validate(el, None)



from flatland import Form, String
from flatland.ext.creditcard import CreditCardNumber, VISA, MASTERCARD


def test_simple():

    class Schema(Form):
        num = CreditCardNumber.named('num')

    data = Schema({})
    assert not data.validate()
    assert data.el('num').errors
    e1 = list(data.el('num').errors)

    data = Schema({'num': 'asdf'})
    assert not data.validate()
    assert data['num'].errors
    assert data['num'].errors != e1
    e2 = list(data['num'].errors)

    data = Schema({'num': '1234'})
    assert not data.validate()
    assert data['num'].errors
    assert data['num'].errors == e2
    e3 = list(data['num'].errors)

    data = Schema({'num': '4100000000000009'})
    assert not data.validate()
    assert data['num'].errors
    assert data['num'].errors != e3
    e4 = list(data['num'].errors)

    data = Schema({'num': '4100000000000001'})
    assert data.validate()
    assert isinstance(data['num'].value, long)
    assert data['num'].u == '4100-0000-0000-0001'


def test_subclass():

    class MyCreditCard(CreditCardNumber):

        class Present(CreditCardNumber.Present):
            missing = u'Yo! You need a %(label)s!'

    class Schema(Form):
        num = MyCreditCard.using(label='Visa/MC Number',
                                 accepted=(VISA, MASTERCARD))
        name = String.using(optional=True)


    data = Schema.from_flat({})
    assert not data.validate()
    assert data['num'].errors
    err = data['num'].errors[0]

    assert err.startswith('Yo!')
    assert 'Visa/MC Number' in err



import __builtin__
import itertools


def element_ancestry(element):
    """Iterates element plus element.parents."""
    return itertools.chain((element,), element.parents)


def search_ancestry(element, predicate):
    """Run *predicate* against element's ancestry.

    :param element: an :class:`~flatland.schema.base.Element`

    :param predicate: a callable function. Will be passed one
      argument, an element.

    Returns the first non-false value emitted by *predicate*.
    AttributeError raised by *predicate* are treated as equivalent to
    a pass.  Any other exception is raised to the caller.

    """
    for obj in element_ancestry(element):
        try:
            transform = predicate(obj)
            if transform:
                return transform
        except AttributeError:
            pass


def find_i18n_function(element, finder):
    """Find i18n form helpers such as ``ugettext``.

    :param element: an :class:`~flatland.schema.base.Element`

    :param finder: a callable function. Will be passed one argument,
      an element.  AttributeError is ignored, allowing
      ``operator.attrgetter('ugettext')`` to be used directly.

    Searches the ancestry of *element* and it's schema with *finder*
    ala :func:`search_ancestry`, falling back to a search against
    ``__builtin__``.

    """
    transformer = search_ancestry(element, finder)
    if transformer:
        return transformer
    try:
        return finder(__builtin__)
    except AttributeError:
        return None
